from ctypes import (
    POINTER,
    c_bool,
    c_byte,
    c_char,
    c_char_p,
    c_double,
    c_float,
    c_int,
    c_int32,
    c_int64,
    c_long,
    c_short,
    c_uint,
    c_ulong,
    c_void_p,
    cdll,
    pointer)
from .definitions.safearray import SafeArray
from .definitions.enumerations import (
    MOT_JogModes,
    MOT_LimitSwitchModes,
    MOT_LimitSwitchSWModes,
    MOT_LimitsSoftwareApproachPolicy,
    MOT_MovementDirections,
    MOT_MovementModes,
    MOT_StopModes,
    MOT_TravelDirection,
    MOT_TravelModes)
from .definitions.structures import (
    MOT_HomingParameters,
    MOT_JogParameters,
    MOT_JoystickParameters,
    MOT_LimitSwitchParameters,
    MOT_PIDLoopEncoderParams,
    MOT_PowerParameters,
    MOT_VelocityParameters,
    TLI_DeviceInfo,
    TLI_HardwareInformation)
from .definitions.kinesisexception import KinesisException


lib_path = "C:/Program Files/Thorlabs/Kinesis/"
device_manager = cdll.LoadLibrary(
    lib_path + "Thorlabs.MotionControl.DeviceManager.dll")

lib = cdll.LoadLibrary(
    lib_path + "Thorlabs.MotionControl.Benchtop.StepperMotor.dll")


# Build the DeviceList.
TLI_BuildDeviceList = lib.TLI_BuildDeviceList
TLI_BuildDeviceList.restype = c_short
TLI_BuildDeviceList.argtypes = []


# Initialize a connection to the Simulation Manager, which must already be running.
TLI_InitializeSimulations = lib.TLI_InitializeSimulations
TLI_InitializeSimulations.restype = c_void_p
TLI_InitializeSimulations.argtypes = []


# Can the device perform a Home.
SBC_CanHome = lib.SBC_CanHome
SBC_CanHome.restype = c_bool
SBC_CanHome.argtypes = [POINTER(c_char), c_short]


# Can this device be moved without Homing.
SBC_CanMoveWithoutHomingFirst = lib.SBC_CanMoveWithoutHomingFirst
SBC_CanMoveWithoutHomingFirst.restype = c_bool
SBC_CanMoveWithoutHomingFirst.argtypes = [POINTER(c_char), c_short]


# Check connection.
SBC_CheckConnection = lib.SBC_CheckConnection
SBC_CheckConnection.restype = c_bool
SBC_CheckConnection.argtypes = [POINTER(c_char)]


# Clears the device message queue.
SBC_ClearMessageQueue = lib.SBC_ClearMessageQueue
SBC_ClearMessageQueue.restype = c_short
SBC_ClearMessageQueue.argtypes = [POINTER(c_char), c_short]


# Disconnect and close the device.
SBC_Close = lib.SBC_Close
SBC_Close.restype = c_short
SBC_Close.argtypes = [POINTER(c_char)]


# Disable the channel so that motor can be moved by hand.
SBC_DisableChannel = lib.SBC_DisableChannel
SBC_DisableChannel.restype = c_short
SBC_DisableChannel.argtypes = [POINTER(c_char), c_short]


# Enable channel for computer control.
SBC_EnableChannel = lib.SBC_EnableChannel
SBC_EnableChannel.restype = c_short
SBC_EnableChannel.argtypes = [POINTER(c_char), c_short]


# Enables the last message monitoring timer.
SBC_EnableLastMsgTimer = lib.SBC_EnableLastMsgTimer
SBC_EnableLastMsgTimer.restype = c_void_p
SBC_EnableLastMsgTimer.argtypes = [POINTER(c_char), c_short, c_bool, c_int32]


# Get the backlash distance setting (used to control hysteresis).
SBC_GetBacklash = lib.SBC_GetBacklash
SBC_GetBacklash.restype = c_long
SBC_GetBacklash.argtypes = [POINTER(c_char), c_short]


# Gets the stepper motor bow index.
SBC_GetBowIndex = lib.SBC_GetBowIndex
SBC_GetBowIndex.restype = c_short
SBC_GetBowIndex.argtypes = [POINTER(c_char), c_short]


# Get calibration file for this motor.
SBC_GetCalibrationFile = lib.SBC_GetCalibrationFile
SBC_GetCalibrationFile.restype = c_bool
SBC_GetCalibrationFile.argtypes = [POINTER(c_char), c_short, POINTER(c_char), c_short]


# Converts a device unit to a real world unit.
SBC_GetDeviceUnitFromRealValue = lib.SBC_GetDeviceUnitFromRealValue
SBC_GetDeviceUnitFromRealValue.restype = c_short
SBC_GetDeviceUnitFromRealValue.argtypes = [POINTER(c_char), c_short, c_double, c_int, c_int]


# Gets the digital output bits.
SBC_GetDigitalOutputs = lib.SBC_GetDigitalOutputs
SBC_GetDigitalOutputs.restype = c_byte
SBC_GetDigitalOutputs.argtypes = [POINTER(c_char), c_short]


# Get the Encoder Counter.
SBC_GetEncoderCounter = lib.SBC_GetEncoderCounter
SBC_GetEncoderCounter.restype = c_long
SBC_GetEncoderCounter.argtypes = [POINTER(c_char), c_short]


# Gets version number of the device firmware.
SBC_GetFirmwareVersion = lib.SBC_GetFirmwareVersion
SBC_GetFirmwareVersion.restype = c_ulong
SBC_GetFirmwareVersion.argtypes = [POINTER(c_char), c_short]


# Gets the hardware information from the device.
SBC_GetHardwareInfo = lib.SBC_GetHardwareInfo
SBC_GetHardwareInfo.restype = c_short
SBC_GetHardwareInfo.argtypes = [POINTER(c_char)]


# Gets the hardware information in a block.
SBC_GetHardwareInfoBlock = lib.SBC_GetHardwareInfoBlock
SBC_GetHardwareInfoBlock.restype = c_short
SBC_GetHardwareInfoBlock.argtypes = [POINTER(c_char)]


# Get the homing parameters.
SBC_GetHomingParamsBlock = lib.SBC_GetHomingParamsBlock
SBC_GetHomingParamsBlock.restype = c_short
SBC_GetHomingParamsBlock.argtypes = [POINTER(c_char), c_short, MOT_HomingParameters]


# Gets the homing velocity.
SBC_GetHomingVelocity = lib.SBC_GetHomingVelocity
SBC_GetHomingVelocity.restype = c_uint
SBC_GetHomingVelocity.argtypes = [POINTER(c_char), c_short]


# Gets the analogue input voltage reading.
SBC_GetInputVoltage = lib.SBC_GetInputVoltage
SBC_GetInputVoltage.restype = c_long
SBC_GetInputVoltage.argtypes = [POINTER(c_char), c_short]


# Gets the jog mode.
SBC_GetJogMode = lib.SBC_GetJogMode
SBC_GetJogMode.restype = c_short
SBC_GetJogMode.argtypes = [POINTER(c_char), c_short, MOT_JogModes, MOT_StopModes]


# Get the jog parameters.
SBC_GetJogParamsBlock = lib.SBC_GetJogParamsBlock
SBC_GetJogParamsBlock.restype = c_short
SBC_GetJogParamsBlock.argtypes = [POINTER(c_char), c_short, MOT_JogParameters, MOT_JogParameters]


# Gets the distance to move when jogging.
SBC_GetJogStepSize = lib.SBC_GetJogStepSize
SBC_GetJogStepSize.restype = c_uint
SBC_GetJogStepSize.argtypes = [POINTER(c_char), c_short]


# Gets the jog velocity parameters.
SBC_GetJogVelParams = lib.SBC_GetJogVelParams
SBC_GetJogVelParams.restype = c_short
SBC_GetJogVelParams.argtypes = [POINTER(c_char), c_short, c_int, c_int]


# Gets the joystick parameters.
SBC_GetJoystickParams = lib.SBC_GetJoystickParams
SBC_GetJoystickParams.restype = c_short
SBC_GetJoystickParams.argtypes = [POINTER(c_char), c_short, MOT_JoystickParameters]


# Gets the limit switch parameters.
SBC_GetLimitSwitchParams = lib.SBC_GetLimitSwitchParams
SBC_GetLimitSwitchParams.restype = c_short
SBC_GetLimitSwitchParams.argtypes = [POINTER(c_char), c_short, MOT_LimitSwitchModes, MOT_LimitSwitchModes, c_uint, c_uint, MOT_LimitSwitchSWModes]


# Get the limit switch parameters.
SBC_GetLimitSwitchParamsBlock = lib.SBC_GetLimitSwitchParamsBlock
SBC_GetLimitSwitchParamsBlock.restype = c_short
SBC_GetLimitSwitchParamsBlock.argtypes = [POINTER(c_char), c_short, MOT_LimitSwitchParameters]


# Sets the motor stage parameters.
SBC_GetMotorParams = lib.SBC_GetMotorParams
SBC_GetMotorParams.restype = c_short
SBC_GetMotorParams.argtypes = [POINTER(c_char), c_short, c_long, c_long, c_float]


# Sets the motor stage parameters.
SBC_GetMotorParamsExt = lib.SBC_GetMotorParamsExt
SBC_GetMotorParamsExt.restype = c_short
SBC_GetMotorParamsExt.argtypes = [POINTER(c_char), c_short, c_double, c_double, c_double]


# Gets the absolute minimum and maximum travel range constants for the current stage.
SBC_GetMotorTravelLimits = lib.SBC_GetMotorTravelLimits
SBC_GetMotorTravelLimits.restype = c_short
SBC_GetMotorTravelLimits.argtypes = [POINTER(c_char), c_short, c_double, c_double]


# Get the motor travel mode.
SBC_GetMotorTravelMode = lib.SBC_GetMotorTravelMode
SBC_GetMotorTravelMode.restype = MOT_TravelModes
SBC_GetMotorTravelMode.argtypes = [POINTER(c_char), c_short]


# Gets the absolute maximum velocity and acceleration constants for the current stage.
SBC_GetMotorVelocityLimits = lib.SBC_GetMotorVelocityLimits
SBC_GetMotorVelocityLimits.restype = c_short
SBC_GetMotorVelocityLimits.argtypes = [POINTER(c_char), c_short, c_double, c_double]


# Gets the move absolute position.
SBC_GetMoveAbsolutePosition = lib.SBC_GetMoveAbsolutePosition
SBC_GetMoveAbsolutePosition.restype = c_int
SBC_GetMoveAbsolutePosition.argtypes = [POINTER(c_char), c_short]


# Gets the move relative distance.
SBC_GetMoveRelativeDistance = lib.SBC_GetMoveRelativeDistance
SBC_GetMoveRelativeDistance.restype = c_int
SBC_GetMoveRelativeDistance.argtypes = [POINTER(c_char), c_short]


# Get the next MessageQueue item if it is available.
SBC_GetNextMessage = lib.SBC_GetNextMessage
SBC_GetNextMessage.restype = c_bool
SBC_GetNextMessage.argtypes = [POINTER(c_char), c_short, c_long, c_long, c_ulong]


# Gets the number of channels in the device.
SBC_GetNumChannels = lib.SBC_GetNumChannels
SBC_GetNumChannels.restype = c_short
SBC_GetNumChannels.argtypes = [POINTER(c_char)]


# Get number of positions.
SBC_GetNumberPositions = lib.SBC_GetNumberPositions
SBC_GetNumberPositions.restype = c_int
SBC_GetNumberPositions.argtypes = [POINTER(c_char), c_short]


# Gets the Encoder PID loop encoder coefficient.
SBC_GetPIDLoopEncoderCoeff = lib.SBC_GetPIDLoopEncoderCoeff
SBC_GetPIDLoopEncoderCoeff.restype = c_double
SBC_GetPIDLoopEncoderCoeff.argtypes = [POINTER(c_char), c_short]


# Gets the Encoder PID loop parameters.
SBC_GetPIDLoopEncoderParams = lib.SBC_GetPIDLoopEncoderParams
SBC_GetPIDLoopEncoderParams.restype = c_short
SBC_GetPIDLoopEncoderParams.argtypes = [POINTER(c_char), c_short, MOT_PIDLoopEncoderParams]


# Get the current position.
SBC_GetPosition = lib.SBC_GetPosition
SBC_GetPosition.restype = c_int
SBC_GetPosition.argtypes = [POINTER(c_char), c_short]


# Get the Position Counter.
SBC_GetPositionCounter = lib.SBC_GetPositionCounter
SBC_GetPositionCounter.restype = c_long
SBC_GetPositionCounter.argtypes = [POINTER(c_char), c_short]


# Sets the power parameters for the stepper motor.
SBC_GetPowerParams = lib.SBC_GetPowerParams
SBC_GetPowerParams.restype = c_short
SBC_GetPowerParams.argtypes = [POINTER(c_char), c_short, MOT_PowerParameters]


# Gets the rack digital output bits.
SBC_GetRackDigitalOutputs = lib.SBC_GetRackDigitalOutputs
SBC_GetRackDigitalOutputs.restype = c_byte
SBC_GetRackDigitalOutputs.argtypes = [POINTER(c_char)]


# Gets the Rack status bits.
SBC_GetRackStatusBits = lib.SBC_GetRackStatusBits
SBC_GetRackStatusBits.restype = c_ulong
SBC_GetRackStatusBits.argtypes = [POINTER(c_char)]


# Converts a device unit to a real world unit.
SBC_GetRealValueFromDeviceUnit = lib.SBC_GetRealValueFromDeviceUnit
SBC_GetRealValueFromDeviceUnit.restype = c_short
SBC_GetRealValueFromDeviceUnit.argtypes = [POINTER(c_char), c_short, c_int, c_double, c_int]


# Gets the software limits mode.
SBC_GetSoftLimitMode = lib.SBC_GetSoftLimitMode
SBC_GetSoftLimitMode.restype = MOT_LimitsSoftwareApproachPolicy
SBC_GetSoftLimitMode.argtypes = [POINTER(c_char), c_short]


# Gets version number of the device software.
SBC_GetSoftwareVersion = lib.SBC_GetSoftwareVersion
SBC_GetSoftwareVersion.restype = c_ulong
SBC_GetSoftwareVersion.argtypes = [POINTER(c_char)]


# Gets the Stepper Motor maximum stage position.
SBC_GetStageAxisMaxPos = lib.SBC_GetStageAxisMaxPos
SBC_GetStageAxisMaxPos.restype = c_int
SBC_GetStageAxisMaxPos.argtypes = [POINTER(c_char), c_short]


# Gets the Stepper Motor minimum stage position.
SBC_GetStageAxisMinPos = lib.SBC_GetStageAxisMinPos
SBC_GetStageAxisMinPos.restype = c_int
SBC_GetStageAxisMinPos.argtypes = [POINTER(c_char), c_short]


# Get the current status bits.
SBC_GetStatusBits = lib.SBC_GetStatusBits
SBC_GetStatusBits.restype = c_ulong
SBC_GetStatusBits.argtypes = [POINTER(c_char), c_short]


# Gets the trigger switch parameter.
SBC_GetTriggerSwitches = lib.SBC_GetTriggerSwitches
SBC_GetTriggerSwitches.restype = c_byte
SBC_GetTriggerSwitches.argtypes = [POINTER(c_char), c_short]


# Gets the move velocity parameters.
SBC_GetVelParams = lib.SBC_GetVelParams
SBC_GetVelParams.restype = c_short
SBC_GetVelParams.argtypes = [POINTER(c_char), c_short, c_int, c_int]


# Get the move velocity parameters.
SBC_GetVelParamsBlock = lib.SBC_GetVelParamsBlock
SBC_GetVelParamsBlock.restype = c_short
SBC_GetVelParamsBlock.argtypes = [POINTER(c_char), c_short, MOT_VelocityParameters, MOT_VelocityParameters]


# Queries if the time since the last message has exceeded the lastMsgTimeout set by SBC_EnableLastMsgTimer(char const * serialNo, bool enable, __int32 lastMsgTimeout ).
SBC_HasLastMsgTimerOverrun = lib.SBC_HasLastMsgTimerOverrun
SBC_HasLastMsgTimerOverrun.restype = c_bool
SBC_HasLastMsgTimerOverrun.argtypes = [POINTER(c_char), c_short]


# Home the device.
SBC_Home = lib.SBC_Home
SBC_Home.restype = c_short
SBC_Home.argtypes = [POINTER(c_char), c_short]


# Sends a command to the device to make it identify iteself.
SBC_Identify = lib.SBC_Identify
SBC_Identify.restype = c_void_p
SBC_Identify.argtypes = [POINTER(c_char), c_short]


# Is a calibration file active for this motor.
SBC_IsCalibrationActive = lib.SBC_IsCalibrationActive
SBC_IsCalibrationActive.restype = c_bool
SBC_IsCalibrationActive.argtypes = [POINTER(c_char), c_short]


# Verifies that the specified channel is valid.
SBC_IsChannelValid = lib.SBC_IsChannelValid
SBC_IsChannelValid.restype = c_bool
SBC_IsChannelValid.argtypes = [POINTER(c_char), c_short]


# Update device with named settings.
SBC_LoadNamedSettings = lib.SBC_LoadNamedSettings
SBC_LoadNamedSettings.restype = c_bool
SBC_LoadNamedSettings.argtypes = [POINTER(c_char), c_short, POINTER(c_char)]


# Update device with stored settings.
SBC_LoadSettings = lib.SBC_LoadSettings
SBC_LoadSettings.restype = c_bool
SBC_LoadSettings.argtypes = [POINTER(c_char), c_short]


# Gets the number of channels available to this device.
SBC_MaxChannelCount = lib.SBC_MaxChannelCount
SBC_MaxChannelCount.restype = c_int
SBC_MaxChannelCount.argtypes = [POINTER(c_char)]


# Gets the MessageQueue size.
SBC_MessageQueueSize = lib.SBC_MessageQueueSize
SBC_MessageQueueSize.restype = c_int
SBC_MessageQueueSize.argtypes = [POINTER(c_char), c_short]


# Moves the device to the position defined in the SetMoveAbsolute command.
SBC_MoveAbsolute = lib.SBC_MoveAbsolute
SBC_MoveAbsolute.restype = c_short
SBC_MoveAbsolute.argtypes = [POINTER(c_char), c_short]


# Start moving at the current velocity in the specified direction.
SBC_MoveAtVelocity = lib.SBC_MoveAtVelocity
SBC_MoveAtVelocity.restype = c_short
SBC_MoveAtVelocity.argtypes = [POINTER(c_char), c_short, MOT_TravelDirection]


# Perform a jog.
SBC_MoveJog = lib.SBC_MoveJog
SBC_MoveJog.restype = c_short
SBC_MoveJog.argtypes = [POINTER(c_char), c_short, MOT_TravelDirection]


# Move the motor by a relative amount.
SBC_MoveRelative = lib.SBC_MoveRelative
SBC_MoveRelative.restype = c_short
SBC_MoveRelative.argtypes = [POINTER(c_char), c_short, c_int]


# Moves the device by a relative distancce defined by SetMoveRelativeDistance.
SBC_MoveRelativeDistance = lib.SBC_MoveRelativeDistance
SBC_MoveRelativeDistance.restype = c_short
SBC_MoveRelativeDistance.argtypes = [POINTER(c_char), c_short]


# Move the device to the specified position (index).
SBC_MoveToPosition = lib.SBC_MoveToPosition
SBC_MoveToPosition.restype = c_short
SBC_MoveToPosition.argtypes = [POINTER(c_char), c_short, c_int]


# Does the device need to be Homed before a move can be performed.
SBC_NeedsHoming = lib.SBC_NeedsHoming
SBC_NeedsHoming.restype = c_bool
SBC_NeedsHoming.argtypes = [POINTER(c_char), c_short]


# Open the device for communications.
SBC_Open = lib.SBC_Open
SBC_Open.restype = c_short
SBC_Open.argtypes = [POINTER(c_char)]


# Persist device settings to device.
SBC_PersistSettings = lib.SBC_PersistSettings
SBC_PersistSettings.restype = c_bool
SBC_PersistSettings.argtypes = [POINTER(c_char), c_short]


# Gets the polling loop duration.
SBC_PollingDuration = lib.SBC_PollingDuration
SBC_PollingDuration.restype = c_long
SBC_PollingDuration.argtypes = [POINTER(c_char), c_short]


# Registers a callback on the message queue.
SBC_RegisterMessageCallback = lib.SBC_RegisterMessageCallback
SBC_RegisterMessageCallback.restype = c_short
SBC_RegisterMessageCallback.argtypes = [POINTER(c_char), c_short, c_void_p]


# Requests the backlash.
SBC_RequestBacklash = lib.SBC_RequestBacklash
SBC_RequestBacklash.restype = c_short
SBC_RequestBacklash.argtypes = [POINTER(c_char), c_short]


# Requests the stepper motor bow index.
SBC_RequestBowIndex = lib.SBC_RequestBowIndex
SBC_RequestBowIndex.restype = c_short
SBC_RequestBowIndex.argtypes = [POINTER(c_char), c_short]


# Requests the digital output bits.
SBC_RequestDigitalOutputs = lib.SBC_RequestDigitalOutputs
SBC_RequestDigitalOutputs.restype = c_short
SBC_RequestDigitalOutputs.argtypes = [POINTER(c_char), c_short]


# Requests the encoder counter.
SBC_RequestEncoderCounter = lib.SBC_RequestEncoderCounter
SBC_RequestEncoderCounter.restype = c_short
SBC_RequestEncoderCounter.argtypes = [POINTER(c_char), c_short]


# Requests the homing parameters.
SBC_RequestHomingParams = lib.SBC_RequestHomingParams
SBC_RequestHomingParams.restype = c_short
SBC_RequestHomingParams.argtypes = [POINTER(c_char), c_short]


# Requests the analogue input voltage reading.
SBC_RequestInputVoltage = lib.SBC_RequestInputVoltage
SBC_RequestInputVoltage.restype = c_short
SBC_RequestInputVoltage.argtypes = [POINTER(c_char), c_short]


# Requests the jog parameters.
SBC_RequestJogParams = lib.SBC_RequestJogParams
SBC_RequestJogParams.restype = c_short
SBC_RequestJogParams.argtypes = [POINTER(c_char), c_short]


# Requests the joystick parameters.
SBC_RequestJoystickParams = lib.SBC_RequestJoystickParams
SBC_RequestJoystickParams.restype = c_short
SBC_RequestJoystickParams.argtypes = [POINTER(c_char), c_short]


# Requests the limit switch parameters.
SBC_RequestLimitSwitchParams = lib.SBC_RequestLimitSwitchParams
SBC_RequestLimitSwitchParams.restype = c_short
SBC_RequestLimitSwitchParams.argtypes = [POINTER(c_char), c_short]


# Requests the position of next absolute move.
SBC_RequestMoveAbsolutePosition = lib.SBC_RequestMoveAbsolutePosition
SBC_RequestMoveAbsolutePosition.restype = c_short
SBC_RequestMoveAbsolutePosition.argtypes = [POINTER(c_char), c_short]


# Requests the relative move distance.
SBC_RequestMoveRelativeDistance = lib.SBC_RequestMoveRelativeDistance
SBC_RequestMoveRelativeDistance.restype = c_short
SBC_RequestMoveRelativeDistance.argtypes = [POINTER(c_char), c_short]


# Requests the Encoder PID loop parameters.
SBC_RequestPIDLoopEncoderParams = lib.SBC_RequestPIDLoopEncoderParams
SBC_RequestPIDLoopEncoderParams.restype = c_short
SBC_RequestPIDLoopEncoderParams.argtypes = [POINTER(c_char), c_short]


# Requests the current position.
SBC_RequestPosition = lib.SBC_RequestPosition
SBC_RequestPosition.restype = c_short
SBC_RequestPosition.argtypes = [POINTER(c_char), c_short]


# Requests the power parameters.
SBC_RequestPowerParams = lib.SBC_RequestPowerParams
SBC_RequestPowerParams.restype = c_short
SBC_RequestPowerParams.argtypes = [POINTER(c_char), c_short]


# Requests the rack digital output bits.
SBC_RequestRackDigitalOutputs = lib.SBC_RequestRackDigitalOutputs
SBC_RequestRackDigitalOutputs.restype = c_short
SBC_RequestRackDigitalOutputs.argtypes = [POINTER(c_char)]


# Requests the Rack status bits be downloaded.
SBC_RequestRackStatusBits = lib.SBC_RequestRackStatusBits
SBC_RequestRackStatusBits.restype = c_short
SBC_RequestRackStatusBits.argtypes = [POINTER(c_char)]


# Requests that all settings are download from device.
SBC_RequestSettings = lib.SBC_RequestSettings
SBC_RequestSettings.restype = c_short
SBC_RequestSettings.argtypes = [POINTER(c_char), c_short]


# Request the status bits which identify the current motor state.
SBC_RequestStatusBits = lib.SBC_RequestStatusBits
SBC_RequestStatusBits.restype = c_short
SBC_RequestStatusBits.argtypes = [POINTER(c_char), c_short]


# Requests the trigger switch parameter.
SBC_RequestTriggerSwitches = lib.SBC_RequestTriggerSwitches
SBC_RequestTriggerSwitches.restype = c_short
SBC_RequestTriggerSwitches.argtypes = [POINTER(c_char), c_short]


# Requests the velocity parameters.
SBC_RequestVelParams = lib.SBC_RequestVelParams
SBC_RequestVelParams.restype = c_short
SBC_RequestVelParams.argtypes = [POINTER(c_char), c_short]


# Reset the rotation modes for a rotational device.
SBC_ResetRotationModes = lib.SBC_ResetRotationModes
SBC_ResetRotationModes.restype = c_short
SBC_ResetRotationModes.argtypes = [POINTER(c_char), c_short]


# Resume suspended move messages.
SBC_ResumeMoveMessages = lib.SBC_ResumeMoveMessages
SBC_ResumeMoveMessages.restype = c_short
SBC_ResumeMoveMessages.argtypes = [POINTER(c_char), c_short]


# Sets the backlash distance (used to control hysteresis).
SBC_SetBacklash = lib.SBC_SetBacklash
SBC_SetBacklash.restype = c_short
SBC_SetBacklash.argtypes = [POINTER(c_char), c_short, c_long]


# Sets the stepper motor bow index.
SBC_SetBowIndex = lib.SBC_SetBowIndex
SBC_SetBowIndex.restype = c_short
SBC_SetBowIndex.argtypes = [POINTER(c_char), c_short, c_short]


# Set the calibration file for this motor.
SBC_SetCalibrationFile = lib.SBC_SetCalibrationFile
SBC_SetCalibrationFile.restype = c_void_p
SBC_SetCalibrationFile.argtypes = [POINTER(c_char), c_short, POINTER(c_char), c_bool]


# Sets the digital output bits.
SBC_SetDigitalOutputs = lib.SBC_SetDigitalOutputs
SBC_SetDigitalOutputs.restype = c_short
SBC_SetDigitalOutputs.argtypes = [POINTER(c_char), c_short, c_byte]


# Sets the motor direction sense.
SBC_SetDirection = lib.SBC_SetDirection
SBC_SetDirection.restype = c_short
SBC_SetDirection.argtypes = [POINTER(c_char), c_short, c_bool]


# Set the Encoder Counter values.
SBC_SetEncoderCounter = lib.SBC_SetEncoderCounter
SBC_SetEncoderCounter.restype = c_short
SBC_SetEncoderCounter.argtypes = [POINTER(c_char), c_short, c_long]


# Set the homing parameters.
SBC_SetHomingParamsBlock = lib.SBC_SetHomingParamsBlock
SBC_SetHomingParamsBlock.restype = c_short
SBC_SetHomingParamsBlock.argtypes = [POINTER(c_char), c_short, MOT_HomingParameters]


# Sets the homing velocity.
SBC_SetHomingVelocity = lib.SBC_SetHomingVelocity
SBC_SetHomingVelocity.restype = c_short
SBC_SetHomingVelocity.argtypes = [POINTER(c_char), c_short, c_uint]


# Sets the jog mode.
SBC_SetJogMode = lib.SBC_SetJogMode
SBC_SetJogMode.restype = c_short
SBC_SetJogMode.argtypes = [POINTER(c_char), c_short, MOT_JogModes, MOT_StopModes]


# Set the jog parameters.
SBC_SetJogParamsBlock = lib.SBC_SetJogParamsBlock
SBC_SetJogParamsBlock.restype = c_short
SBC_SetJogParamsBlock.argtypes = [POINTER(c_char), c_short, MOT_JogParameters, MOT_JogParameters]


# Sets the distance to move on jogging.
SBC_SetJogStepSize = lib.SBC_SetJogStepSize
SBC_SetJogStepSize.restype = c_short
SBC_SetJogStepSize.argtypes = [POINTER(c_char), c_short, c_uint]


# Sets jog velocity parameters.
SBC_SetJogVelParams = lib.SBC_SetJogVelParams
SBC_SetJogVelParams.restype = c_short
SBC_SetJogVelParams.argtypes = [POINTER(c_char), c_short, c_int, c_int]


# Sets the joystick parameters.
SBC_SetJoystickParams = lib.SBC_SetJoystickParams
SBC_SetJoystickParams.restype = c_short
SBC_SetJoystickParams.argtypes = [POINTER(c_char), c_short, MOT_JoystickParameters]


# Sets the limit switch parameters.
SBC_SetLimitSwitchParams = lib.SBC_SetLimitSwitchParams
SBC_SetLimitSwitchParams.restype = c_short
SBC_SetLimitSwitchParams.argtypes = [POINTER(c_char), c_short, MOT_LimitSwitchModes, MOT_LimitSwitchModes, c_uint, c_uint, MOT_LimitSwitchSWModes]


# Set the limit switch parameters.
SBC_SetLimitSwitchParamsBlock = lib.SBC_SetLimitSwitchParamsBlock
SBC_SetLimitSwitchParamsBlock.restype = c_short
SBC_SetLimitSwitchParamsBlock.argtypes = [POINTER(c_char), c_short, MOT_LimitSwitchParameters]


# Sets the software limits mode.
SBC_SetLimitsSoftwareApproachPolicy = lib.SBC_SetLimitsSoftwareApproachPolicy
SBC_SetLimitsSoftwareApproachPolicy.restype = c_void_p
SBC_SetLimitsSoftwareApproachPolicy.argtypes = [POINTER(c_char), c_short, MOT_LimitsSoftwareApproachPolicy]


# Sets the motor stage parameters.
SBC_SetMotorParams = lib.SBC_SetMotorParams
SBC_SetMotorParams.restype = c_short
SBC_SetMotorParams.argtypes = [POINTER(c_char), c_short, c_long, c_long, c_float]


# Sets the motor stage parameters.
SBC_SetMotorParamsExt = lib.SBC_SetMotorParamsExt
SBC_SetMotorParamsExt.restype = c_short
SBC_SetMotorParamsExt.argtypes = [POINTER(c_char), c_short, c_double, c_double, c_double]


# Sets the absolute minimum and maximum travel range constants for the current stage.
SBC_SetMotorTravelLimits = lib.SBC_SetMotorTravelLimits
SBC_SetMotorTravelLimits.restype = c_short
SBC_SetMotorTravelLimits.argtypes = [POINTER(c_char), c_short, c_double, c_double]


# Set the motor travel mode.
SBC_SetMotorTravelMode = lib.SBC_SetMotorTravelMode
SBC_SetMotorTravelMode.restype = c_short
SBC_SetMotorTravelMode.argtypes = [POINTER(c_char), c_short, MOT_TravelModes]


# Sets the absolute maximum velocity and acceleration constants for the current stage.
SBC_SetMotorVelocityLimits = lib.SBC_SetMotorVelocityLimits
SBC_SetMotorVelocityLimits.restype = c_short
SBC_SetMotorVelocityLimits.argtypes = [POINTER(c_char), c_short, c_double, c_double]


# Sets the move absolute position.
SBC_SetMoveAbsolutePosition = lib.SBC_SetMoveAbsolutePosition
SBC_SetMoveAbsolutePosition.restype = c_short
SBC_SetMoveAbsolutePosition.argtypes = [POINTER(c_char), c_short, c_int]


# Sets the move relative distance.
SBC_SetMoveRelativeDistance = lib.SBC_SetMoveRelativeDistance
SBC_SetMoveRelativeDistance.restype = c_short
SBC_SetMoveRelativeDistance.argtypes = [POINTER(c_char), c_short, c_int]


# Sets the Encoder PID loop encoder coefficient.
SBC_SetPIDLoopEncoderCoeff = lib.SBC_SetPIDLoopEncoderCoeff
SBC_SetPIDLoopEncoderCoeff.restype = c_short
SBC_SetPIDLoopEncoderCoeff.argtypes = [POINTER(c_char), c_short, c_double]


# Sets the Encoder PID loop parameters.
SBC_SetPIDLoopEncoderParams = lib.SBC_SetPIDLoopEncoderParams
SBC_SetPIDLoopEncoderParams.restype = c_short
SBC_SetPIDLoopEncoderParams.argtypes = [POINTER(c_char), c_short, MOT_PIDLoopEncoderParams]


# Set the Position Counter.
SBC_SetPositionCounter = lib.SBC_SetPositionCounter
SBC_SetPositionCounter.restype = c_short
SBC_SetPositionCounter.argtypes = [POINTER(c_char), c_short, c_long]


# Sets the power parameters for the stepper motor.
SBC_SetPowerParams = lib.SBC_SetPowerParams
SBC_SetPowerParams.restype = c_short
SBC_SetPowerParams.argtypes = [POINTER(c_char), c_short, MOT_PowerParameters]


# Sets the rack digital output bits.
SBC_SetRackDigitalOutputs = lib.SBC_SetRackDigitalOutputs
SBC_SetRackDigitalOutputs.restype = c_short
SBC_SetRackDigitalOutputs.argtypes = [POINTER(c_char), c_byte]


# Set the rotation modes for a rotational device.
SBC_SetRotationModes = lib.SBC_SetRotationModes
SBC_SetRotationModes.restype = c_short
SBC_SetRotationModes.argtypes = [POINTER(c_char), c_short, MOT_MovementModes, MOT_MovementDirections]


# Sets the stage axis position limits.
SBC_SetStageAxisLimits = lib.SBC_SetStageAxisLimits
SBC_SetStageAxisLimits.restype = c_short
SBC_SetStageAxisLimits.argtypes = [POINTER(c_char), c_short, c_int, c_int]


# Sets the trigger switch parameter.
SBC_SetTriggerSwitches = lib.SBC_SetTriggerSwitches
SBC_SetTriggerSwitches.restype = c_short
SBC_SetTriggerSwitches.argtypes = [POINTER(c_char), c_short, c_byte]


# Sets the move velocity parameters.
SBC_SetVelParams = lib.SBC_SetVelParams
SBC_SetVelParams.restype = c_short
SBC_SetVelParams.argtypes = [POINTER(c_char), c_short, c_int, c_int]


# Set the move velocity parameters.
SBC_SetVelParamsBlock = lib.SBC_SetVelParamsBlock
SBC_SetVelParamsBlock.restype = c_short
SBC_SetVelParamsBlock.argtypes = [POINTER(c_char), c_short, MOT_VelocityParameters, MOT_VelocityParameters]


# Starts the internal polling loop which continuously requests position and status.
SBC_StartPolling = lib.SBC_StartPolling
SBC_StartPolling.restype = c_bool
SBC_StartPolling.argtypes = [POINTER(c_char), c_short, c_int]


# Stop the current move immediately (with risk of losing track of position).
SBC_StopImmediate = lib.SBC_StopImmediate
SBC_StopImmediate.restype = c_short
SBC_StopImmediate.argtypes = [POINTER(c_char), c_short]


# Stops the internal polling loop.
SBC_StopPolling = lib.SBC_StopPolling
SBC_StopPolling.restype = c_void_p
SBC_StopPolling.argtypes = [POINTER(c_char), c_short]


# Stop the current move using the current velocity profile.
SBC_StopProfiled = lib.SBC_StopProfiled
SBC_StopProfiled.restype = c_short
SBC_StopProfiled.argtypes = [POINTER(c_char), c_short]


# Suspend automatic messages at ends of moves.
SBC_SuspendMoveMessages = lib.SBC_SuspendMoveMessages
SBC_SuspendMoveMessages.restype = c_short
SBC_SuspendMoveMessages.argtypes = [POINTER(c_char), c_short]


# Gets the time in milliseconds since tha last message was received from the device.
SBC_TimeSinceLastMsgReceived = lib.SBC_TimeSinceLastMsgReceived
SBC_TimeSinceLastMsgReceived.restype = c_bool
SBC_TimeSinceLastMsgReceived.argtypes = [POINTER(c_char), c_short, c_int64]


# Determines if we can uses PID loop encoding.
SBC_UsesPIDLoopEncoding = lib.SBC_UsesPIDLoopEncoding
SBC_UsesPIDLoopEncoding.restype = c_bool
SBC_UsesPIDLoopEncoding.argtypes = [POINTER(c_char), c_short]


# Get the next MessageQueue item if it is available.
SBC_WaitForMessage = lib.SBC_WaitForMessage
SBC_WaitForMessage.restype = c_bool
SBC_WaitForMessage.argtypes = [POINTER(c_char), c_short, c_long, c_long, c_ulong]


# Get the device information from the USB port.
TLI_GetDeviceInfo = lib.TLI_GetDeviceInfo
TLI_GetDeviceInfo.restype = c_short
TLI_GetDeviceInfo.argtypes = [POINTER(c_char), POINTER(c_char), TLI_DeviceInfo]


# Get the entire contents of the device list.
TLI_GetDeviceList = lib.TLI_GetDeviceList
TLI_GetDeviceList.restype = c_short
TLI_GetDeviceList.argtypes = [SafeArray]


# Get the contents of the device list which match the supplied typeID.
TLI_GetDeviceListByType = lib.TLI_GetDeviceListByType
TLI_GetDeviceListByType.restype = c_short
TLI_GetDeviceListByType.argtypes = [SafeArray, c_int]


# Get the contents of the device list which match the supplied typeID.
TLI_GetDeviceListByTypeExt = lib.TLI_GetDeviceListByTypeExt
TLI_GetDeviceListByTypeExt.restype = c_short
TLI_GetDeviceListByTypeExt.argtypes = [POINTER(c_char), c_ulong, c_int]


# Get the contents of the device list which match the supplied typeIDs.
TLI_GetDeviceListByTypes = lib.TLI_GetDeviceListByTypes
TLI_GetDeviceListByTypes.restype = c_short
TLI_GetDeviceListByTypes.argtypes = [SafeArray, c_int, c_int]


# Get the contents of the device list which match the supplied typeIDs.
TLI_GetDeviceListByTypesExt = lib.TLI_GetDeviceListByTypesExt
TLI_GetDeviceListByTypesExt.restype = c_short
TLI_GetDeviceListByTypesExt.argtypes = [POINTER(c_char), c_ulong, c_int, c_int]


# Get the entire contents of the device list.
TLI_GetDeviceListExt = lib.TLI_GetDeviceListExt
TLI_GetDeviceListExt.restype = c_short
TLI_GetDeviceListExt.argtypes = [POINTER(c_char), c_ulong]


# Gets the device list size.
TLI_GetDeviceListSize = lib.TLI_GetDeviceListSize
TLI_GetDeviceListSize.restype = c_short
TLI_GetDeviceListSize.argtypes = []




from ctypes import (
    POINTER,
    c_bool,
    c_byte,
    c_char,
    c_char_p,
    c_int,
    c_int16,
    c_int32,
    c_long,
    c_short,
    c_ulong,
    c_void_p,
    cdll,
    pointer)
from .definitions.safearray import SafeArray
from .definitions.enumerations import (
    KPZ_WheelChangeRate,
    KPZ_WheelDirectionSense,
    KPZ_WheelMode,
    KSG_TriggerPortMode,
    KSG_TriggerPortPolarity,
    PZ_ControlModeTypes,
    PZ_InputSourceFlags)
from .definitions.structures import (
    KSG_TriggerConfig,
    PPC_IOSettings,
    PPC_PIDConsts,
    PPC_PIDCriteria,
    TLI_DeviceInfo,
    TLI_HardwareInformation)
from .definitions.kinesisexception import KinesisException


lib_path = "C:/Program Files/Thorlabs/Kinesis/"
device_manager = cdll.LoadLibrary(
    lib_path + "Thorlabs.MotionControl.DeviceManager.dll")

lib = cdll.LoadLibrary(
    lib_path + "Thorlabs.MotionControl.IntegratedPrecisionPiezo.DLL")


# Build the DeviceList.
TLI_BuildDeviceList = lib.TLI_BuildDeviceList
TLI_BuildDeviceList.restype = c_short
TLI_BuildDeviceList.argtypes = []


# Initialize a connection to the Simulation Manager, which must already be running.
TLI_InitializeSimulations = lib.TLI_InitializeSimulations
TLI_InitializeSimulations.restype = c_void_p
TLI_InitializeSimulations.argtypes = []


# Determine if the device front panel can be locked.
IPP_CanDeviceLockFrontPanel = lib.IPP_CanDeviceLockFrontPanel
IPP_CanDeviceLockFrontPanel.restype = c_bool
IPP_CanDeviceLockFrontPanel.argtypes = [POINTER(c_char)]


# Check connection.
IPP_CheckConnection = lib.IPP_CheckConnection
IPP_CheckConnection.restype = c_bool
IPP_CheckConnection.argtypes = [POINTER(c_char)]


# Clears the device message queue.
IPP_ClearMessageQueue = lib.IPP_ClearMessageQueue
IPP_ClearMessageQueue.restype = c_short
IPP_ClearMessageQueue.argtypes = [POINTER(c_char)]


# Disconnect and close the device.
IPP_Close = lib.IPP_Close
IPP_Close.restype = c_void_p
IPP_Close.argtypes = [POINTER(c_char)]


# Disable the channel so that motor can be moved by hand.
IPP_DisableChannel = lib.IPP_DisableChannel
IPP_DisableChannel.restype = c_short
IPP_DisableChannel.argtypes = [POINTER(c_char)]


# Tells the device that it is being disconnected.
IPP_Disconnect = lib.IPP_Disconnect
IPP_Disconnect.restype = c_short
IPP_Disconnect.argtypes = [POINTER(c_char)]


# Enable channel for computer control.
IPP_EnableChannel = lib.IPP_EnableChannel
IPP_EnableChannel.restype = c_short
IPP_EnableChannel.argtypes = [POINTER(c_char)]


# Gets version number of the device firmware.
IPP_GetFirmwareVersion = lib.IPP_GetFirmwareVersion
IPP_GetFirmwareVersion.restype = c_ulong
IPP_GetFirmwareVersion.argtypes = [POINTER(c_char)]


# Query if the device front panel locked.
IPP_GetFrontPanelLocked = lib.IPP_GetFrontPanelLocked
IPP_GetFrontPanelLocked.restype = c_bool
IPP_GetFrontPanelLocked.argtypes = [POINTER(c_char)]


# Gets the hardware information from the device.
IPP_GetHardwareInfo = lib.IPP_GetHardwareInfo
IPP_GetHardwareInfo.restype = c_short
IPP_GetHardwareInfo.argtypes = [POINTER(c_char)]


# Gets the hardware information in a block.
IPP_GetHardwareInfoBlock = lib.IPP_GetHardwareInfoBlock
IPP_GetHardwareInfoBlock.restype = c_short
IPP_GetHardwareInfoBlock.argtypes = [POINTER(c_char)]


# Gets the PPC IO Settings.
IPP_GetIOSettings = lib.IPP_GetIOSettings
IPP_GetIOSettings.restype = c_short
IPP_GetIOSettings.argtypes = [POINTER(c_char), PPC_IOSettings]


# Get the MMI Parameters for the Integrated Precision Piezo.
IPP_GetMMIParams = lib.IPP_GetMMIParams
IPP_GetMMIParams.restype = c_short
IPP_GetMMIParams.argtypes = [POINTER(c_char), KPZ_WheelMode, KPZ_WheelChangeRate, c_int32, KPZ_WheelDirectionSense, c_int32, c_int32, c_int16, c_int16, c_int16]


# Gets the maximum output voltage (140V) in units of 1 tenth of a volt
IPP_GetMaxOutputVoltage = lib.IPP_GetMaxOutputVoltage
IPP_GetMaxOutputVoltage.restype = c_short
IPP_GetMaxOutputVoltage.argtypes = [POINTER(c_char)]


# Gets the minimum output voltage (-10V) in units of 1 tenth of a volt.
IPP_GetMinOutputVoltage = lib.IPP_GetMinOutputVoltage
IPP_GetMinOutputVoltage.restype = c_short
IPP_GetMinOutputVoltage.argtypes = [POINTER(c_char)]


# Get the next MessageQueue item if it is available.
IPP_GetNextMessage = lib.IPP_GetNextMessage
IPP_GetNextMessage.restype = c_bool
IPP_GetNextMessage.argtypes = [POINTER(c_char), c_long, c_long, c_ulong]


# Gets the set Output Voltage.
IPP_GetOutputVoltage = lib.IPP_GetOutputVoltage
IPP_GetOutputVoltage.restype = c_short
IPP_GetOutputVoltage.argtypes = [POINTER(c_char)]


# Gets the PPC PID Constants.
IPP_GetPIDConsts = lib.IPP_GetPIDConsts
IPP_GetPIDConsts.restype = c_short
IPP_GetPIDConsts.argtypes = [POINTER(c_char), c_byte, PPC_PIDConsts]


# Gets the PID Criteria.
IPP_GetPIDCriteria = lib.IPP_GetPIDCriteria
IPP_GetPIDCriteria.restype = c_short
IPP_GetPIDCriteria.argtypes = [POINTER(c_char), c_byte, PPC_PIDCriteria]


# Gets the current position Please note this is non linear
IPP_GetPosition = lib.IPP_GetPosition
IPP_GetPosition.restype = c_short
IPP_GetPosition.argtypes = [POINTER(c_char)]


# Gets the Position Control Mode.
IPP_GetPositionControlMode = lib.IPP_GetPositionControlMode
IPP_GetPositionControlMode.restype = PZ_ControlModeTypes
IPP_GetPositionControlMode.argtypes = [POINTER(c_char)]


# Gets version number of the device software.
IPP_GetSoftwareVersion = lib.IPP_GetSoftwareVersion
IPP_GetSoftwareVersion.restype = c_ulong
IPP_GetSoftwareVersion.argtypes = [POINTER(c_char)]


# Get the current status bits.
IPP_GetStatusBits = lib.IPP_GetStatusBits
IPP_GetStatusBits.restype = c_ulong
IPP_GetStatusBits.argtypes = [POINTER(c_char)]


# Get the Trigger Configuration Parameters.
IPP_GetTriggerConfigParams = lib.IPP_GetTriggerConfigParams
IPP_GetTriggerConfigParams.restype = c_short
IPP_GetTriggerConfigParams.argtypes = [POINTER(c_char), KSG_TriggerPortMode, KSG_TriggerPortPolarity, KSG_TriggerPortMode, KSG_TriggerPortPolarity, c_int32, c_int32, c_int16]


# Gets the trigger configuration parameters block.
IPP_GetTriggerConfigParamsBlock = lib.IPP_GetTriggerConfigParamsBlock
IPP_GetTriggerConfigParamsBlock.restype = c_short
IPP_GetTriggerConfigParamsBlock.argtypes = [POINTER(c_char), KSG_TriggerConfig]


# Gets the control voltage source.
IPP_GetVoltageSource = lib.IPP_GetVoltageSource
IPP_GetVoltageSource.restype = PZ_InputSourceFlags
IPP_GetVoltageSource.argtypes = [POINTER(c_char)]


# Sends a command to the device to make it identify iteself.
IPP_Identify = lib.IPP_Identify
IPP_Identify.restype = c_void_p
IPP_Identify.argtypes = [POINTER(c_char)]


# Update device with named settings.
IPP_LoadNamedSettings = lib.IPP_LoadNamedSettings
IPP_LoadNamedSettings.restype = c_bool
IPP_LoadNamedSettings.argtypes = [POINTER(c_char), POINTER(c_char)]


# Update device with stored settings.
IPP_LoadSettings = lib.IPP_LoadSettings
IPP_LoadSettings.restype = c_bool
IPP_LoadSettings.argtypes = [POINTER(c_char)]


# Gets the MessageQueue size.
IPP_MessageQueueSize = lib.IPP_MessageQueueSize
IPP_MessageQueueSize.restype = c_int
IPP_MessageQueueSize.argtypes = [POINTER(c_char)]


# Open the device for communications.
IPP_Open = lib.IPP_Open
IPP_Open.restype = c_short
IPP_Open.argtypes = [POINTER(c_char)]


# Persist device settings to device.
IPP_PersistSettings = lib.IPP_PersistSettings
IPP_PersistSettings.restype = c_bool
IPP_PersistSettings.argtypes = [POINTER(c_char)]


# Gets the polling loop duration.
IPP_PollingDuration = lib.IPP_PollingDuration
IPP_PollingDuration.restype = c_long
IPP_PollingDuration.argtypes = [POINTER(c_char)]


# Registers a callback on the message queue.
IPP_RegisterMessageCallback = lib.IPP_RegisterMessageCallback
IPP_RegisterMessageCallback.restype = c_short
IPP_RegisterMessageCallback.argtypes = [POINTER(c_char), c_void_p]


# Ask the device if its front panel is locked.
IPP_RequestFrontPanelLocked = lib.IPP_RequestFrontPanelLocked
IPP_RequestFrontPanelLocked.restype = c_short
IPP_RequestFrontPanelLocked.argtypes = [POINTER(c_char)]


# Requests the curent PPC IO Setting.
IPP_RequestIOSettings = lib.IPP_RequestIOSettings
IPP_RequestIOSettings.restype = c_bool
IPP_RequestIOSettings.argtypes = [POINTER(c_char)]


# Request that the MMI Parameters for the Integrated Precision Piezo be read from the device.
IPP_RequestMMIParams = lib.IPP_RequestMMIParams
IPP_RequestMMIParams.restype = c_bool
IPP_RequestMMIParams.argtypes = [POINTER(c_char)]


# Requests the maximum output voltage be read from the device.
IPP_RequestOutputVoltage = lib.IPP_RequestOutputVoltage
IPP_RequestOutputVoltage.restype = c_bool
IPP_RequestOutputVoltage.argtypes = [POINTER(c_char)]


# Requests the PID Constants.
IPP_RequestPIDConsts = lib.IPP_RequestPIDConsts
IPP_RequestPIDConsts.restype = c_short
IPP_RequestPIDConsts.argtypes = [POINTER(c_char), c_byte]


# Requests the PID Criteria.
IPP_RequestPIDCriteria = lib.IPP_RequestPIDCriteria
IPP_RequestPIDCriteria.restype = c_short
IPP_RequestPIDCriteria.argtypes = [POINTER(c_char), c_byte]


# Gets the current Closed Loop position.
IPP_RequestPosition = lib.IPP_RequestPosition
IPP_RequestPosition.restype = c_bool
IPP_RequestPosition.argtypes = [POINTER(c_char)]


# Requests that the Position Control Mode be read from the device.
IPP_RequestPositionControlMode = lib.IPP_RequestPositionControlMode
IPP_RequestPositionControlMode.restype = c_bool
IPP_RequestPositionControlMode.argtypes = [POINTER(c_char)]


# Requests that all settings are download from device.
IPP_RequestSettings = lib.IPP_RequestSettings
IPP_RequestSettings.restype = c_short
IPP_RequestSettings.argtypes = [POINTER(c_char)]


# Requests the status bits and position.
IPP_RequestStatus = lib.IPP_RequestStatus
IPP_RequestStatus.restype = c_short
IPP_RequestStatus.argtypes = [POINTER(c_char)]


# Request the status bits which identify the current device state.
IPP_RequestStatusBits = lib.IPP_RequestStatusBits
IPP_RequestStatusBits.restype = c_short
IPP_RequestStatusBits.argtypes = [POINTER(c_char)]


# Requests the trigger config parameters
IPP_RequestTriggerConfigParams = lib.IPP_RequestTriggerConfigParams
IPP_RequestTriggerConfigParams.restype = c_short
IPP_RequestTriggerConfigParams.argtypes = [POINTER(c_char)]


# Requests that the current input voltage source be read from the device.
IPP_RequestVoltageSource = lib.IPP_RequestVoltageSource
IPP_RequestVoltageSource.restype = c_bool
IPP_RequestVoltageSource.argtypes = [POINTER(c_char)]


# Sets the voltage output to zero and defines the ensuing actuator position az zero.
IPP_ResetParameters = lib.IPP_ResetParameters
IPP_ResetParameters.restype = c_short
IPP_ResetParameters.argtypes = [POINTER(c_char)]


# Sets the device front panel lock state.
IPP_SetFrontPanelLock = lib.IPP_SetFrontPanelLock
IPP_SetFrontPanelLock.restype = c_short
IPP_SetFrontPanelLock.argtypes = [POINTER(c_char), c_bool]


# Sets the PPC IO Setting.
IPP_SetIOSettings = lib.IPP_SetIOSettings
IPP_SetIOSettings.restype = c_short
IPP_SetIOSettings.argtypes = [POINTER(c_char), PPC_IOSettings]


# Set the MMI Parameters for the Integrated PrecisionPiezo.
IPP_SetMMIParams = lib.IPP_SetMMIParams
IPP_SetMMIParams.restype = c_short
IPP_SetMMIParams.argtypes = [POINTER(c_char), KPZ_WheelMode, KPZ_WheelChangeRate, c_int32, KPZ_WheelDirectionSense, c_int32, c_int32, c_int16, c_int16, c_int16]


# Sets the output voltage.
IPP_SetOutputVoltage = lib.IPP_SetOutputVoltage
IPP_SetOutputVoltage.restype = c_short
IPP_SetOutputVoltage.argtypes = [POINTER(c_char), c_short]


# Sets the PPC PID Constants.
IPP_SetPIDConsts = lib.IPP_SetPIDConsts
IPP_SetPIDConsts.restype = c_short
IPP_SetPIDConsts.argtypes = [POINTER(c_char), PPC_PIDConsts]


# Sets the PID Criteria.
IPP_SetPIDCriteria = lib.IPP_SetPIDCriteria
IPP_SetPIDCriteria.restype = c_short
IPP_SetPIDCriteria.argtypes = [POINTER(c_char), PPC_PIDCriteria]


# Sets the position when in closed loop mode.
IPP_SetPosition = lib.IPP_SetPosition
IPP_SetPosition.restype = c_short
IPP_SetPosition.argtypes = [POINTER(c_char), c_short]


# Sets the Position Control Mode.
IPP_SetPositionControlMode = lib.IPP_SetPositionControlMode
IPP_SetPositionControlMode.restype = c_short
IPP_SetPositionControlMode.argtypes = [POINTER(c_char), PZ_ControlModeTypes]


# Set the Trigger Configuration Parameters.
IPP_SetTriggerConfigParams = lib.IPP_SetTriggerConfigParams
IPP_SetTriggerConfigParams.restype = c_short
IPP_SetTriggerConfigParams.argtypes = [POINTER(c_char), KSG_TriggerPortMode, KSG_TriggerPortPolarity, KSG_TriggerPortMode, KSG_TriggerPortPolarity, c_int32, c_int32, c_int16]


# Sets the trigger configuration parameters block.
IPP_SetTriggerConfigParamsBlock = lib.IPP_SetTriggerConfigParamsBlock
IPP_SetTriggerConfigParamsBlock.restype = c_short
IPP_SetTriggerConfigParamsBlock.argtypes = [POINTER(c_char), KSG_TriggerConfig]


# Sets the control voltage source.
IPP_SetVoltageSource = lib.IPP_SetVoltageSource
IPP_SetVoltageSource.restype = c_short
IPP_SetVoltageSource.argtypes = [POINTER(c_char), PZ_InputSourceFlags]


# Performs a Set Zero operation.
IPP_SetZero = lib.IPP_SetZero
IPP_SetZero.restype = c_short
IPP_SetZero.argtypes = [POINTER(c_char)]


# Starts the internal polling loop which continuously requests position and status.
IPP_StartPolling = lib.IPP_StartPolling
IPP_StartPolling.restype = c_bool
IPP_StartPolling.argtypes = [POINTER(c_char), c_int]


# Stops the internal polling loop.
IPP_StopPolling = lib.IPP_StopPolling
IPP_StopPolling.restype = c_void_p
IPP_StopPolling.argtypes = [POINTER(c_char)]


# Get the next MessageQueue item if it is available.
IPP_WaitForMessage = lib.IPP_WaitForMessage
IPP_WaitForMessage.restype = c_bool
IPP_WaitForMessage.argtypes = [POINTER(c_char), c_long, c_long, c_ulong]


# Get the device information from the USB port.
TLI_GetDeviceInfo = lib.TLI_GetDeviceInfo
TLI_GetDeviceInfo.restype = c_short
TLI_GetDeviceInfo.argtypes = [POINTER(c_char), POINTER(c_char), TLI_DeviceInfo]


# Get the entire contents of the device list.
TLI_GetDeviceList = lib.TLI_GetDeviceList
TLI_GetDeviceList.restype = c_short
TLI_GetDeviceList.argtypes = [SafeArray]


# Get the contents of the device list which match the supplied typeID.
TLI_GetDeviceListByType = lib.TLI_GetDeviceListByType
TLI_GetDeviceListByType.restype = c_short
TLI_GetDeviceListByType.argtypes = [SafeArray, c_int]


# Get the contents of the device list which match the supplied typeID.
TLI_GetDeviceListByTypeExt = lib.TLI_GetDeviceListByTypeExt
TLI_GetDeviceListByTypeExt.restype = c_short
TLI_GetDeviceListByTypeExt.argtypes = [POINTER(c_char), c_ulong, c_int]


# Get the contents of the device list which match the supplied typeIDs.
TLI_GetDeviceListByTypes = lib.TLI_GetDeviceListByTypes
TLI_GetDeviceListByTypes.restype = c_short
TLI_GetDeviceListByTypes.argtypes = [SafeArray, c_int, c_int]


# Get the contents of the device list which match the supplied typeIDs.
TLI_GetDeviceListByTypesExt = lib.TLI_GetDeviceListByTypesExt
TLI_GetDeviceListByTypesExt.restype = c_short
TLI_GetDeviceListByTypesExt.argtypes = [POINTER(c_char), c_ulong, c_int, c_int]


# Get the entire contents of the device list.
TLI_GetDeviceListExt = lib.TLI_GetDeviceListExt
TLI_GetDeviceListExt.restype = c_short
TLI_GetDeviceListExt.argtypes = [POINTER(c_char), c_ulong]


# Gets the device list size.
TLI_GetDeviceListSize = lib.TLI_GetDeviceListSize
TLI_GetDeviceListSize.restype = c_short
TLI_GetDeviceListSize.argtypes = []




from ctypes import (
    POINTER,
    c_bool,
    c_char,
    c_char_p,
    c_int,
    c_int32,
    c_int64,
    c_long,
    c_short,
    c_ulong,
    c_void_p,
    cdll,
    pointer)
from .definitions.safearray import SafeArray
from .definitions.enumerations import (
    LS_DisplayUnits,
    LS_InputSourceFlags)
from .definitions.structures import (
    TLI_DeviceInfo,
    TLI_HardwareInformation)
from .definitions.kinesisexception import KinesisException


lib_path = "C:/Program Files/Thorlabs/Kinesis/"
device_manager = cdll.LoadLibrary(
    lib_path + "Thorlabs.MotionControl.DeviceManager.dll")

lib = cdll.LoadLibrary(
    lib_path + "Thorlabs.MotionControl.TCube.LaserSource.DLL")


# Build the DeviceList.
TLI_BuildDeviceList = lib.TLI_BuildDeviceList
TLI_BuildDeviceList.restype = c_short
TLI_BuildDeviceList.argtypes = []


# Initialize a connection to the Simulation Manager, which must already be running.
TLI_InitializeSimulations = lib.TLI_InitializeSimulations
TLI_InitializeSimulations.restype = c_void_p
TLI_InitializeSimulations.argtypes = []


# Check connection.
LS_CheckConnection = lib.LS_CheckConnection
LS_CheckConnection.restype = c_bool
LS_CheckConnection.argtypes = [POINTER(c_char)]


# Clears the device message queue.
LS_ClearMessageQueue = lib.LS_ClearMessageQueue
LS_ClearMessageQueue.restype = c_void_p
LS_ClearMessageQueue.argtypes = [POINTER(c_char)]


# Disconnect and close the device.
LS_Close = lib.LS_Close
LS_Close.restype = c_void_p
LS_Close.argtypes = [POINTER(c_char)]


# Disable laser.
LS_Disable = lib.LS_Disable
LS_Disable.restype = c_short
LS_Disable.argtypes = [POINTER(c_char)]


# Switch laser off.
LS_DisableOutput = lib.LS_DisableOutput
LS_DisableOutput.restype = c_short
LS_DisableOutput.argtypes = [POINTER(c_char)]


# Enable laser for computer control.
LS_Enable = lib.LS_Enable
LS_Enable.restype = c_short
LS_Enable.argtypes = [POINTER(c_char)]


# Enables the last message monitoring timer.
LS_EnableLastMsgTimer = lib.LS_EnableLastMsgTimer
LS_EnableLastMsgTimer.restype = c_void_p
LS_EnableLastMsgTimer.argtypes = [POINTER(c_char), c_bool, c_int32]


# Switch laser on.
LS_EnableOutput = lib.LS_EnableOutput
LS_EnableOutput.restype = c_short
LS_EnableOutput.argtypes = [POINTER(c_char)]


# Gets the control input source.
LS_GetControlSource = lib.LS_GetControlSource
LS_GetControlSource.restype = LS_InputSourceFlags
LS_GetControlSource.argtypes = [POINTER(c_char)]


# Gets current Current reading.
LS_GetCurrentReading = lib.LS_GetCurrentReading
LS_GetCurrentReading.restype = c_long
LS_GetCurrentReading.argtypes = [POINTER(c_char)]


# Gets the hardware display units.
LS_GetDisplayUnits = lib.LS_GetDisplayUnits
LS_GetDisplayUnits.restype = LS_DisplayUnits
LS_GetDisplayUnits.argtypes = [POINTER(c_char)]


# Gets version number of the device firmware.
LS_GetFirmwareVersion = lib.LS_GetFirmwareVersion
LS_GetFirmwareVersion.restype = c_ulong
LS_GetFirmwareVersion.argtypes = [POINTER(c_char)]


# Gets the hardware information from the device.
LS_GetHardwareInfo = lib.LS_GetHardwareInfo
LS_GetHardwareInfo.restype = c_short
LS_GetHardwareInfo.argtypes = [POINTER(c_char)]


# Gets the hardware information in a block.
LS_GetHardwareInfoBlock = lib.LS_GetHardwareInfoBlock
LS_GetHardwareInfoBlock.restype = c_short
LS_GetHardwareInfoBlock.argtypes = [POINTER(c_char)]


# Gets the Interlock State.
LS_GetInterlockState = lib.LS_GetInterlockState
LS_GetInterlockState.restype = c_byte
LS_GetInterlockState.argtypes = [POINTER(c_char)]


# Gets the LED brightness.
LS_GetLEDBrightness = lib.LS_GetLEDBrightness
LS_GetLEDBrightness.restype = c_long
LS_GetLEDBrightness.argtypes = [POINTER(c_char)]


# Gets the max power and current limits for the device.
LS_GetLimits = lib.LS_GetLimits
LS_GetLimits.restype = c_short
LS_GetLimits.argtypes = [POINTER(c_char), c_long, c_long]


# Get the next MessageQueue item.
LS_GetNextMessage = lib.LS_GetNextMessage
LS_GetNextMessage.restype = c_bool
LS_GetNextMessage.argtypes = [POINTER(c_char), c_long, c_long, c_ulong]


# Gets current power reading.
LS_GetPowerReading = lib.LS_GetPowerReading
LS_GetPowerReading.restype = c_long
LS_GetPowerReading.argtypes = [POINTER(c_char)]


# Gets the output power currently set.
LS_GetPowerSet = lib.LS_GetPowerSet
LS_GetPowerSet.restype = c_long
LS_GetPowerSet.argtypes = [POINTER(c_char)]


# Gets version number of the device software.
LS_GetSoftwareVersion = lib.LS_GetSoftwareVersion
LS_GetSoftwareVersion.restype = c_ulong
LS_GetSoftwareVersion.argtypes = [POINTER(c_char)]


# Get the current status bits.
LS_GetStatusBits = lib.LS_GetStatusBits
LS_GetStatusBits.restype = c_ulong
LS_GetStatusBits.argtypes = [POINTER(c_char)]


# Gets the operating wavelength.
LS_GetWavelength = lib.LS_GetWavelength
LS_GetWavelength.restype = c_long
LS_GetWavelength.argtypes = [POINTER(c_char)]


# Queries if the time since the last message has exceeded the lastMsgTimeout set by LS_EnableLastMsgTimer(char const * serialNo, bool enable, __int32 lastMsgTimeout ).
LS_HasLastMsgTimerOverrun = lib.LS_HasLastMsgTimerOverrun
LS_HasLastMsgTimerOverrun.restype = c_bool
LS_HasLastMsgTimerOverrun.argtypes = [POINTER(c_char)]


# Sends a command to the device to make it identify iteself.
LS_Identify = lib.LS_Identify
LS_Identify.restype = c_void_p
LS_Identify.argtypes = [POINTER(c_char)]


# Update device with named settings.
LS_LoadNamedSettings = lib.LS_LoadNamedSettings
LS_LoadNamedSettings.restype = c_bool
LS_LoadNamedSettings.argtypes = [POINTER(c_char), POINTER(c_char)]


# Update device with stored settings.
LS_LoadSettings = lib.LS_LoadSettings
LS_LoadSettings.restype = c_bool
LS_LoadSettings.argtypes = [POINTER(c_char)]


# Gets the MessageQueue size.
LS_MessageQueueSize = lib.LS_MessageQueueSize
LS_MessageQueueSize.restype = c_int
LS_MessageQueueSize.argtypes = [POINTER(c_char)]


# Open the device for communications.
LS_Open = lib.LS_Open
LS_Open.restype = c_short
LS_Open.argtypes = [POINTER(c_char)]


# persist the devices current settings.
LS_PersistSettings = lib.LS_PersistSettings
LS_PersistSettings.restype = c_bool
LS_PersistSettings.argtypes = [POINTER(c_char)]


# Gets the polling loop duration.
LS_PollingDuration = lib.LS_PollingDuration
LS_PollingDuration.restype = c_long
LS_PollingDuration.argtypes = [POINTER(c_char)]


# Registers a callback on the message queue.
LS_RegisterMessageCallback = lib.LS_RegisterMessageCallback
LS_RegisterMessageCallback.restype = c_void_p
LS_RegisterMessageCallback.argtypes = [POINTER(c_char), c_void_p]


# Gets the control input source.
LS_RequestControlSource = lib.LS_RequestControlSource
LS_RequestControlSource.restype = c_short
LS_RequestControlSource.argtypes = [POINTER(c_char)]


# Requests the hardware display units.
LS_RequestDisplayUnits = lib.LS_RequestDisplayUnits
LS_RequestDisplayUnits.restype = c_short
LS_RequestDisplayUnits.argtypes = [POINTER(c_char)]


# Requests the LED Brightness.
LS_RequestLEDBrightness = lib.LS_RequestLEDBrightness
LS_RequestLEDBrightness.restype = c_short
LS_RequestLEDBrightness.argtypes = [POINTER(c_char)]


# Requests the limits MaxPower and MaxCurrent.
LS_RequestLimits = lib.LS_RequestLimits
LS_RequestLimits.restype = c_short
LS_RequestLimits.argtypes = [POINTER(c_char)]


# Request power and current readings.
LS_RequestReadings = lib.LS_RequestReadings
LS_RequestReadings.restype = c_short
LS_RequestReadings.argtypes = [POINTER(c_char)]


# Sets the output power.
LS_RequestSetPower = lib.LS_RequestSetPower
LS_RequestSetPower.restype = c_short
LS_RequestSetPower.argtypes = [POINTER(c_char)]


# Requests that all settings are download from device.
LS_RequestSettings = lib.LS_RequestSettings
LS_RequestSettings.restype = c_short
LS_RequestSettings.argtypes = [POINTER(c_char)]


# Requests the state quantities (actual power, current and status).
LS_RequestStatus = lib.LS_RequestStatus
LS_RequestStatus.restype = c_short
LS_RequestStatus.argtypes = [POINTER(c_char)]


# Request the status bits which identify the current device state.
LS_RequestStatusBits = lib.LS_RequestStatusBits
LS_RequestStatusBits.restype = c_short
LS_RequestStatusBits.argtypes = [POINTER(c_char)]


# Requests the device Wavelength.
LS_RequestWavelength = lib.LS_RequestWavelength
LS_RequestWavelength.restype = c_short
LS_RequestWavelength.argtypes = [POINTER(c_char)]


# Sets the control input source.
LS_SetControlSource = lib.LS_SetControlSource
LS_SetControlSource.restype = c_short
LS_SetControlSource.argtypes = [POINTER(c_char), LS_InputSourceFlags]


# Sets the hardware display units.
LS_SetDisplayUnits = lib.LS_SetDisplayUnits
LS_SetDisplayUnits.restype = c_short
LS_SetDisplayUnits.argtypes = [POINTER(c_char), LS_DisplayUnits]


# Sets the LED brightness.
LS_SetLEDBrightness = lib.LS_SetLEDBrightness
LS_SetLEDBrightness.restype = c_short
LS_SetLEDBrightness.argtypes = [POINTER(c_char), c_short]


# Sets the output power.
LS_SetPower = lib.LS_SetPower
LS_SetPower.restype = c_short
LS_SetPower.argtypes = [POINTER(c_char), c_long]


# Starts the internal polling loop which continuously requests position and status.
LS_StartPolling = lib.LS_StartPolling
LS_StartPolling.restype = c_bool
LS_StartPolling.argtypes = [POINTER(c_char), c_int]


# Stops the internal polling loop.
LS_StopPolling = lib.LS_StopPolling
LS_StopPolling.restype = c_void_p
LS_StopPolling.argtypes = [POINTER(c_char)]


# Gets the time in milliseconds since tha last message was received from the device.
LS_TimeSinceLastMsgReceived = lib.LS_TimeSinceLastMsgReceived
LS_TimeSinceLastMsgReceived.restype = c_bool
LS_TimeSinceLastMsgReceived.argtypes = [POINTER(c_char), c_int64]


# Wait for next MessageQueue item.
LS_WaitForMessage = lib.LS_WaitForMessage
LS_WaitForMessage.restype = c_bool
LS_WaitForMessage.argtypes = [POINTER(c_char), c_long, c_long, c_ulong]


# Get the device information from the USB port.
TLI_GetDeviceInfo = lib.TLI_GetDeviceInfo
TLI_GetDeviceInfo.restype = c_short
TLI_GetDeviceInfo.argtypes = [POINTER(c_char), POINTER(c_char), TLI_DeviceInfo]


# Get the entire contents of the device list.
TLI_GetDeviceList = lib.TLI_GetDeviceList
TLI_GetDeviceList.restype = c_short
TLI_GetDeviceList.argtypes = [SafeArray]


# Get the contents of the device list which match the supplied typeID.
TLI_GetDeviceListByType = lib.TLI_GetDeviceListByType
TLI_GetDeviceListByType.restype = c_short
TLI_GetDeviceListByType.argtypes = [SafeArray, c_int]


# Get the contents of the device list which match the supplied typeID.
TLI_GetDeviceListByTypeExt = lib.TLI_GetDeviceListByTypeExt
TLI_GetDeviceListByTypeExt.restype = c_short
TLI_GetDeviceListByTypeExt.argtypes = [POINTER(c_char), c_ulong, c_int]


# Get the contents of the device list which match the supplied typeIDs.
TLI_GetDeviceListByTypes = lib.TLI_GetDeviceListByTypes
TLI_GetDeviceListByTypes.restype = c_short
TLI_GetDeviceListByTypes.argtypes = [SafeArray, c_int, c_int]


# Get the contents of the device list which match the supplied typeIDs.
TLI_GetDeviceListByTypesExt = lib.TLI_GetDeviceListByTypesExt
TLI_GetDeviceListByTypesExt.restype = c_short
TLI_GetDeviceListByTypesExt.argtypes = [POINTER(c_char), c_ulong, c_int, c_int]


# Get the entire contents of the device list.
TLI_GetDeviceListExt = lib.TLI_GetDeviceListExt
TLI_GetDeviceListExt.restype = c_short
TLI_GetDeviceListExt.argtypes = [POINTER(c_char), c_ulong]


# Gets the device list size.
TLI_GetDeviceListSize = lib.TLI_GetDeviceListSize
TLI_GetDeviceListSize.restype = c_short
TLI_GetDeviceListSize.argtypes = []




from ctypes import (
    Structure,
    c_bool,
    c_byte,
    c_char,
    c_float,
    c_int,
    c_int16,
    c_int32,
    c_long,
    c_short,
    c_uint,
    c_uint16,
    c_uint32,
    c_uint8,
    c_ulong,
    c_ushort)
from .enumerations import (
    BNT_BNCTriggerModes,
    BNT_CurrentLimit,
    BNT_FeedbackSignalSelection,
    BNT_OutputLowPassFilter,
    FF_IOModes,
    FF_SignalModes,
    HubAnalogueModes,
    KIM_Channels,
    KIM_DirectionSense,
    KIM_FBSignalMode,
    KIM_JogMode,
    KIM_JoystickModes,
    KIM_LimitSwitchModes,
    KIM_TravelDirection,
    KIM_TrigModes,
    KIM_TrigPolarities,
    KLDTriggerMode,
    KLD_TrigPolarity,
    KLS_Polarity,
    KLS_TriggerMode,
    KMOT_TriggerPortMode,
    KMOT_TriggerPortPolarity,
    KMOT_WheelMode,
    KNA_HighOutputVoltageRoute,
    KNA_HighVoltageRange,
    KNA_LowOutputVoltageRoute,
    KNA_LowVoltageRange,
    KNA_TIARange,
    KNA_TriggerPolarity,
    KNA_TriggerPortMode,
    KNA_WheelAdjustRate,
    KPC_MonitorOutputMode,
    KPC_TriggerPortMode,
    KPC_TriggerPortPolarity,
    KPZ_TriggerPortMode,
    KPZ_TriggerPortPolarity,
    KPZ_WheelChangeRate,
    KPZ_WheelDirectionSense,
    KPZ_WheelMode,
    KSC_TriggerPolarity,
    KSC_TriggerPortMode,
    MOD_AuxIOPortMode,
    MOD_IOPortMode,
    MOD_IOPortSource,
    MOD_Monitor_Variable,
    MOT_ButtonModes,
    MOT_CurrentLoopPhases,
    MOT_DirectionSense,
    MOT_HomeLimitSwitchDirection,
    MOT_JogModes,
    MOT_LimitSwitchModes,
    MOT_LimitSwitchSWModes,
    MOT_MotorTypes,
    MOT_PID_LoopMode,
    MOT_RasterScanMovePattern,
    MOT_RasterScanMoveTriggerMode,
    MOT_StopModes,
    MOT_TravelDirection,
    MOT_TriggerInputConfigModes,
    MOT_TriggerInputSource,
    MOT_TriggerOutputConfigModes,
    MOT_TriggerPolarity,
    MOT_VelocityProfileModes,
    MPC_IOModes,
    MPC_SignalModes,
    NT_CircleAdjustment,
    NT_CircleDiameterMode,
    NT_LowPassFrequency,
    NT_OddOrEven,
    NT_OutputVoltageRoute,
    NT_TIARangeMode,
    NT_UnderOrOver,
    NT_VoltageRange,
    PCC_DerivFilterState,
    PCC_DisplayIntensity,
    PCC_FeedbackPolarity,
    PCC_IOFeedbackSourceDefinition,
    PCC_IOOutputBandwidth,
    PCC_IOOutputMode,
    PCC_NotchFilterChannel,
    PCC_NotchFilterState,
    PPC_IOControlMode,
    PZ_JogModes,
    PZ_OutputLUTModes,
    QD_FilterEnable,
    QD_KPA_TrigModes,
    QD_KPA_TrigPolarities,
    QD_LowVoltageRoute,
    QD_OpenLoopHoldValues,
    TSG_DisplayModes,
    TSG_Hub_Analogue_Modes)


class BNT_IO_Settings(Structure):
    _fields_ = [
        ("amplifierCurrentLimit", BNT_CurrentLimit),
        ("amplifierLowPassFilter", BNT_OutputLowPassFilter),
        ("BNCtriggerOrLowVoltageOut", BNT_BNCTriggerModes),
        ("channel", c_long),
        ("feedbackSignal", BNT_FeedbackSignalSelection),
    ]


class FF_IOSettings(Structure):
    _fields_ = [
        ("ADCspeedValue", c_uint),
        ("digIO1OperMode", FF_IOModes),
        ("digIO1PulseWidth", c_uint),
        ("digIO1SignalMode", FF_SignalModes),
        ("digIO2OperMode", FF_IOModes),
        ("digIO2PulseWidth", c_uint),
        ("digIO2SignalMode", FF_SignalModes),
        ("reserved1", c_int),
        ("reserved", c_uint),
        ("transitTime", c_uint),
    ]


class KIM_DriveOPParameters(Structure):
    _fields_ = [
        ("maxVoltage", c_int16),
        ("stepAcceleration", c_int32),
        ("stepRate", c_int32),
    ]


class KIM_FeedbackSigParams(Structure):
    _fields_ = [
        ("encoderConst", c_int32),
        ("feedbackSignalMode", KIM_FBSignalMode),
        ("reserved1", c_int16),
        ("reserved2", c_int16),
        ("reserved3", c_int16),
        ("reserved4", c_int16),
    ]


class KIM_HomeParameters(Structure):
    _fields_ = [
        ("homeDirection", KIM_TravelDirection),
        ("homeLimitSwitch", KIM_TravelDirection),
        ("homeOffset", c_int32),
        ("homeStepRate", c_int32),
    ]


class KIM_JogParameters(Structure):
    _fields_ = [
        ("jogMode", KIM_JogMode),
        ("jogStepAcceleration", c_int32),
        ("jogStepRate", c_int32),
        ("jogStepSizeFwd", c_int32),
        ("jogStepSizeRev", c_int32),
    ]


class KIM_LimitSwitchParameters(Structure):
    _fields_ = [
        ("forwardLimit", KIM_LimitSwitchModes),
        ("reverseLimit", KIM_LimitSwitchModes),
        ("stageID", c_int16),
    ]


class KIM_MMIChannelParameters(Structure):
    _fields_ = [
        ("presentPos1", c_int32),
        ("presentPos2", c_int32),
    ]


class KIM_MMIParameters(Structure):
    _fields_ = [
        ("directionSense", KIM_DirectionSense),
        ("displayIntensity", c_int16),
        ("joystickMode", KIM_JoystickModes),
        ("maxStepRate", c_int32),
    ]


class KIM_Status(Structure):
    _fields_ = [
        ("encoderCount", c_int32),
        ("position", c_int32),
        ("statusBits", c_uint32),
    ]


class KIM_TrigIOConfig(Structure):
    _fields_ = [
        ("trig1Mode", KIM_TrigModes),
        ("trig1Polarity", KIM_TrigPolarities),
        ("trig2Mode", KIM_TrigModes),
        ("trig2Polarity", KIM_TrigPolarities),
        ("trigChannel1", KIM_Channels),
        ("trigChannel2", KIM_Channels),
    ]


class KIM_TrigParamsParameters(Structure):
    _fields_ = [
        ("intervalFwd", c_int32),
        ("intervalRev", c_int32),
        ("numberOfCycles", c_int32),
        ("numberOfPulsesFwd", c_int32),
        ("numberOfPulsesRev", c_int32),
        ("pusleWidth", c_int32),
        ("reserved1", c_int16),
        ("reserved2", c_int16),
        ("reserved3", c_int16),
        ("reserved4", c_int16),
        ("reserved5", c_int16),
        ("reserved6", c_int16),
        ("startPosFwd", c_int32),
        ("startPosRev", c_int32),
    ]


class KLD_MMIParams(Structure):
    _fields_ = [
        ("displayIntensity", c_int16),
        ("reserved1", c_int16),
        ("reserved2", c_int16),
        ("reserved3", c_int16),
    ]


class KLD_TrigIOParams(Structure):
    _fields_ = [
        ("mode1", KLDTriggerMode),
        ("mode2", KLDTriggerMode),
        ("polarity1", KLD_TrigPolarity),
        ("polarity2", KLD_TrigPolarity),
        ("reserved1", c_int16),
        ("reserved2", c_int16),
    ]


class KLS_MMIParams(Structure):
    _fields_ = [
        ("displayIntensity", c_int16),
        ("reserved1", c_int16),
        ("reserved2", c_int16),
        ("reserved3", c_int16),
    ]


class KLS_TrigIOParams(Structure):
    _fields_ = [
        ("mode1", KLS_TriggerMode),
        ("mode2", KLS_TriggerMode),
        ("polarity1", KLS_Polarity),
        ("polarity2", KLS_Polarity),
        ("reserved1", c_int16),
        ("reserved2", c_int16),
    ]


class KMOT_MMIParams(Structure):
    _fields_ = [
        ("DisplayDimIntensity", c_int16),
        ("DisplayIntensity", c_int16),
        ("DisplayTimeout", c_int16),
        ("PresentPos1", c_int32),
        ("PresentPos2", c_int32),
        ("reserved1", c_int16),
        ("reserved2", c_int16),
        ("reserved3", c_int16),
        ("reserved4", c_int16),
        ("WheelAcceleration", c_int32),
        ("WheelDirectionSense", MOT_DirectionSense),
        ("WheelMaxVelocity", c_int32),
        ("WheelMode", KMOT_WheelMode),
    ]


class KMOT_TriggerConfig(Structure):
    _fields_ = [
        ("reserved1", c_int16),
        ("reserved2", c_int16),
        ("reserved3", c_int16),
        ("reserved4", c_int16),
        ("reserved5", c_int16),
        ("reserved6", c_int16),
        ("Trigger1Mode", KMOT_TriggerPortMode),
        ("Trigger1Polarity", KMOT_TriggerPortPolarity),
        ("Trigger2Mode", KMOT_TriggerPortMode),
        ("Trigger2Polarity", KMOT_TriggerPortPolarity),
    ]


class KMOT_TriggerParams(Structure):
    _fields_ = [
        ("Cyclecount", c_int32),
        ("reserved1", c_int32),
        ("reserved2", c_int32),
        ("reserved3", c_int32),
        ("reserved4", c_int32),
        ("reserved5", c_int32),
        ("reserved6", c_int32),
        ("TriggerIntervalFwd", c_int32),
        ("TriggerIntervalRev", c_int32),
        ("TriggerPulseCountFwd", c_int32),
        ("TriggerPulseCountRev", c_int32),
        ("TriggerPulseWidth", c_int32),
        ("TriggerStartPositionFwd", c_int32),
        ("TriggerStartPositionRev", c_int32),
    ]


class KNA_FeedbackLoopConstants(Structure):
    _fields_ = [
        ("integralTerm", c_short),
        ("proportionalTerm", c_short),
    ]


class KNA_IOSettings(Structure):
    _fields_ = [
        ("highVoltageOutputRoute", KNA_HighOutputVoltageRoute),
        ("highVoltageOutRange", KNA_HighVoltageRange),
        ("lowVoltageOutputRoute", KNA_LowOutputVoltageRoute),
        ("lowVoltageOutRange", KNA_LowVoltageRange),
    ]


class KNA_MMIParams(Structure):
    _fields_ = [
        ("DisplayIntensity", c_uint16),
        ("reserved1", c_int16),
        ("reserved2", c_int16),
        ("reserved3", c_int16),
        ("reserved4", c_int16),
        ("reserved5", c_int16),
        ("reserved6", c_int16),
        ("WheelAdjustRate", KNA_WheelAdjustRate),
    ]


class KNA_TIARangeParameters(Structure):
    _fields_ = [
        ("changeToOddOrEven", NT_OddOrEven),
        ("downLimit", c_long),
        ("mode", NT_TIARangeMode),
        ("newRange", KNA_TIARange),
        ("settleSamples", c_short),
        ("upLimit", c_long),
    ]


class KNA_TIAReading(Structure):
    _fields_ = [
        ("absoluteReading", c_float),
        ("relativeReading", c_long),
        ("selectRange", KNA_TIARange),
        ("underOrOverRead", NT_UnderOrOver),
    ]


class KNA_TriggerConfig(Structure):
    _fields_ = [
        ("reserved1", c_int16),
        ("reserved2", c_int16),
        ("reserved3", c_int16),
        ("reserved4", c_int16),
        ("Trigger1Mode", KNA_TriggerPortMode),
        ("Trigger1Polarity", KNA_TriggerPolarity),
        ("Trigger2Mode", KNA_TriggerPortMode),
        ("Trigger2Polarity", KNA_TriggerPolarity),
        ("unused1", c_int16),
        ("unused2", c_int16),
    ]


class KPC_MMIParams(Structure):
    _fields_ = [
        ("DisplayDimIntensity", c_int16),
        ("DisplayIntensity", c_int16),
        ("DisplayTimeout", c_int16),
        ("JoystickDirectionSense", KPZ_WheelChangeRate),
        ("JoystickMode", KPZ_WheelMode),
        ("PositionStep", c_int16),
        ("PresentPos1", c_int16),
        ("PresentPos2", c_int16),
        ("PresentVolt1", c_int16),
        ("PresentVolt2", c_int16),
        ("reserved1", c_int16),
        ("reserved2", c_int16),
        ("reserved3", c_int16),
        ("reserved4", c_int16),
        ("VoltageAdjustRate", KPZ_WheelChangeRate),
        ("VoltageStep", c_int16),
    ]


class KPC_TriggerConfig(Structure):
    _fields_ = [
        ("LowerLimit", c_int32),
        ("MonitorFilterFrequency", c_int16),
        ("MonitorOutput", KPC_MonitorOutputMode),
        ("MonitorOutputSoftwareValue", c_int32),
        ("reserved1", c_int16),
        ("reserved2", c_int16),
        ("reserved3", c_int16),
        ("SmoothingSamples", c_int16),
        ("Trigger1Mode", KPC_TriggerPortMode),
        ("Trigger1Polarity", KPC_TriggerPortPolarity),
        ("Trigger2Mode", KPC_TriggerPortMode),
        ("Trigger2Polarity", KPC_TriggerPortPolarity),
        ("UpperLimit", c_int32),
    ]


class KPZ_MMIParams(Structure):
    _fields_ = [
        ("DisplayDimIntensity", c_int16),
        ("DisplayIntensity", c_int16),
        ("DisplayTimeout", c_int16),
        ("JoystickDirectionSense", KPZ_WheelDirectionSense),
        ("JoystickMode", KPZ_WheelMode),
        ("PresentPos1", c_int32),
        ("PresentPos2", c_int32),
        ("reserved1", c_int16),
        ("reserved2", c_int16),
        ("reserved3", c_int16),
        ("reserved4", c_int16),
        ("VoltageAdjustRate", KPZ_WheelChangeRate),
        ("VoltageStep", c_int32),
    ]


class KPZ_TriggerConfig(Structure):
    _fields_ = [
        ("reserved1", c_int16),
        ("reserved2", c_int16),
        ("reserved3", c_int16),
        ("reserved4", c_int16),
        ("reserved5", c_int16),
        ("reserved6", c_int16),
        ("Trigger1Mode", KPZ_TriggerPortMode),
        ("Trigger1Polarity", KPZ_TriggerPortPolarity),
        ("Trigger2Mode", KPZ_TriggerPortMode),
        ("Trigger2Polarity", KPZ_TriggerPortPolarity),
    ]


class KSC_MMIParams(Structure):
    _fields_ = [
        ("DisplayDimIntensity", c_int16),
        ("DisplayIntensity", c_int16),
        ("DisplayTimeout", c_int16),
        ("reserved1", c_int16),
        ("reserved2", c_int16),
        ("reserved3", c_int16),
        ("reserved4", c_int16),
        ("unused1", c_int16),
        ("unused2", c_int16),
        ("unused3", c_int16),
        ("unused4", c_int16),
        ("unused5", c_int16),
        ("unused6", c_int16),
        ("unused7", c_int16),
        ("unused8", c_int16),
        ("unused9", c_int16),
        ("unused10", c_int16),
    ]


class KSC_TriggerConfig(Structure):
    _fields_ = [
        ("reserved1", c_int16),
        ("reserved2", c_int16),
        ("reserved3", c_int16),
        ("reserved4", c_int16),
        ("reserved5", c_int16),
        ("reserved6", c_int16),
        ("Trigger1Mode", KSC_TriggerPortMode),
        ("Trigger1Polarity", KSC_TriggerPolarity),
        ("Trigger2Mode", KSC_TriggerPortMode),
        ("Trigger2Polarity", KSC_TriggerPolarity),
    ]


class KSG_TriggerConfig(Structure):
    _fields_ = [
        ("LowerLimit", c_int32),
        ("reserved1", c_int16),
        ("reserved2", c_int16),
        ("reserved3", c_int16),
        ("reserved4", c_int16),
        ("reserved5", c_int16),
        ("reserved6", c_int16),
        ("Trigger1Mode", KPZ_TriggerPortMode),
        ("Trigger1Polarity", KPZ_TriggerPortPolarity),
        ("Trigger2Mode", KPZ_TriggerPortMode),
        ("Trigger2Polarity", KPZ_TriggerPortPolarity),
        ("UpperLimit", c_int32),
    ]


class MOD_AnalogMonitorConfigurationParameters(Structure):
    _fields_ = [
        ("MonitorNo", c_long),
        ("MonitorVar", MOD_Monitor_Variable),
        ("MotorChannelNo", c_long),
        ("Offset", c_long),
        ("Scale", c_long),
    ]


class MOD_AuxIOPortConfigurationParameters(Structure):
    _fields_ = [
        ("Mode", MOD_AuxIOPortMode),
        ("PortNo", c_long),
        ("SWState", c_long),
    ]


class MOD_AuxIOPortConfigurationSetParameters(Structure):
    _fields_ = [
        ("Mode", MOD_AuxIOPortMode),
        ("PortNo", c_long),
        ("SWState", c_long),
    ]


class MOD_IOPortConfigurationParameters(Structure):
    _fields_ = [
        ("Mode", MOD_IOPortMode),
        ("PortNo", c_long),
        ("Source", MOD_IOPortSource),
    ]


class MOT_BVC_ScanParams(Structure):
    _fields_ = [
        ("enable", c_bool),
        ("scanAmplitude", c_ulong),
        ("scanCenterPosition", c_long),
        ("scanFrequency", c_ulong),
    ]


class MOT_BrushlessCurrentLoopParameters(Structure):
    _fields_ = [
        ("phase", MOT_CurrentLoopPhases),
        ("proportionalGain", c_long),
        ("integralGain", c_long),
        ("integralLimit", c_long),
        ("deadErrorBand", c_long),
        ("feedForward", c_long),
        ("notUsed", c_long),
        ("lastNotUsed", c_long),
    ]


class MOT_BrushlessElectricOutputParameters(Structure):
    _fields_ = [
        ("continuousCurrentLimit", c_long),
        ("excessEnergyLimit", c_long),
        ("motorSignalLimit", c_short),
        ("motorSignalBias", c_short),
        ("notUsed", c_long),
        ("lastNotUsed", c_long),
    ]


class MOT_BrushlessPositionLoopParameters(Structure):
    _fields_ = [
        ("proportionalGain", c_long),
        ("integralGain", c_long),
        ("integralLimit", c_ulong),
        ("differentialGain", c_long),
        ("derivativeRecalculationTime", c_long),
        ("factorForOutput", c_long),
        ("velocityFeedForward", c_long),
        ("accelerationFeedForward", c_long),
        ("positionErrorLimit", c_ulong),
        ("notUsed", c_long),
        ("lastNotUsed", c_long),
    ]


class MOT_BrushlessTrackSettleParameters(Structure):
    _fields_ = [
        ("time", c_long),
        ("settledError", c_long),
        ("maxTrackingError", c_long),
        ("notUsed", c_long),
        ("lastNotUsed", c_long),
    ]


class MOT_ButtonParameters(Structure):
    _fields_ = [
        ("buttonMode", MOT_ButtonModes),
        ("leftButtonPosition", c_int),
        ("rightButtonPosition", c_int),
        ("timeout", c_long),
        ("unused", c_long),
    ]


class MOT_ChannelPosition(Structure):
    _fields_ = [
        ("channelNumber", c_long),
        ("position", c_int),
    ]


class MOT_DC_PIDParameters(Structure):
    _fields_ = [
        ("differentialGain", c_int),
        ("integralGain", c_int),
        ("integralLimit", c_int),
        ("parameterFilter", c_long),
        ("proportionalGain", c_int),
    ]


class MOT_EncoderResolutionParams(Structure):
    _fields_ = [
        ("enconderResolutionFraction", c_long),
        ("encoderResolutionWholeNumber", c_ulong),
        ("unused1", c_long),
        ("unused2", c_long),
        ("unused3", c_long),
    ]


class MOT_HomingParameters(Structure):
    _fields_ = [
        ("direction", MOT_TravelDirection),
        ("limitSwitch", MOT_HomeLimitSwitchDirection),
        ("offsetDistance", c_uint),
        ("velocity", c_uint),
    ]


class MOT_VelocityParameters(Structure):
    _fields_ = [
        ("minVelocity", c_int),
        ("acceleration", c_int),
        ("maxVelocity", c_int),
    ]


class MOT_JogParameters(Structure):
    _fields_ = [
        ("mode", MOT_JogModes),
        ("stepSize", c_uint),
        ("velParams", MOT_VelocityParameters),
        ("stopMode", MOT_StopModes),
    ]


class MOT_JoystickParameters(Structure):
    _fields_ = [
        ("lowGearMaxVelocity", c_ulong),
        ("highGearMaxVelocity", c_ulong),
        ("lowGearAcceleration", c_ulong),
        ("highGearAcceleration", c_ulong),
        ("directionSense", MOT_TravelDirection),
    ]


class MOT_LCDMoveParams(Structure):
    _fields_ = [
        ("JogAccn", c_int),
        ("JogMaxVel", c_int),
        ("JogMode", MOT_JogModes),
        ("PresentPosition1", c_int),
        ("PresentPosition2", c_int),
        ("PresentPosition3", c_int),
        ("reserved1", c_long),
        ("reserved2", c_long),
        ("reserved3", c_long),
        ("reserved4", c_long),
        ("reserved5", c_long),
        ("reserved6", c_long),
        ("reserved7", c_long),
        ("reserved8", c_long),
        ("reserved9", c_long),
        ("reserved10", c_long),
        ("StepSize", c_uint),
        ("StopMode", MOT_StopModes),
    ]


class MOT_LCDParams(Structure):
    _fields_ = [
        ("DisplayDimIntensity", c_long),
        ("DisplayIntensity", c_long),
        ("DisplayTimeout", c_long),
        ("JSSensitivity", c_long),
        ("reserved1", c_long),
        ("reserved2", c_long),
        ("reserved3", c_long),
        ("reserved4", c_long),
        ("reserved5", c_long),
        ("reserved6", c_long),
        ("reserved7", c_long),
        ("reserved8", c_long),
        ("reserved9", c_long),
        ("reserved10", c_long),
    ]


class MOT_LimitSwitchParameters(Structure):
    _fields_ = [
        ("anticlockwiseHardwareLimit", MOT_LimitSwitchModes),
        ("anticlockwisePosition", c_ulong),
        ("clockwiseHardwareLimit", MOT_LimitSwitchModes),
        ("clockwisePosition", MOT_LimitSwitchSWModes),
    ]


class MOT_PIDLoopEncoderParams(Structure):
    _fields_ = [
        ("differentialGain", c_int),
        ("integralGain", c_int),
        ("loopMode", MOT_PID_LoopMode),
        ("PIDOutputLimit", c_int),
        ("PIDTolerance", c_int),
        ("proportionalGain", c_int),
    ]


class MOT_PotentiometerStep(Structure):
    _fields_ = [
        ("thresholdDeflection", c_long),
        ("velocity", c_ulong),
    ]


class MOT_PotentiometerSteps(Structure):
    _fields_ = [
        ("potentionmeterStepParameters1", MOT_PotentiometerStep),
        ("potentionmeterStepParameters2", MOT_PotentiometerStep),
        ("potentionmeterStepParameters3", MOT_PotentiometerStep),
        ("potentionmeterStepParameters4", MOT_PotentiometerStep),
    ]


class MOT_PowerParameters(Structure):
    _fields_ = [
        ("movePercentage", c_long),
        ("restPercentage", c_long),
    ]


class MOT_RasterScanMoveAxisParams(Structure):
    _fields_ = [
        ("ChannelNo", c_long),
        ("CycleCount", c_long),
        ("DwellTime", c_long),
        ("RelativeDistance", c_long),
        ("StartPos", c_long),
    ]


class MOT_RasterScanMoveParams(Structure):
    _fields_ = [
        ("AxisMovement1", MOT_RasterScanMoveAxisParams),
        ("AxisMovement2", MOT_RasterScanMoveAxisParams),
        ("ScanPattern", MOT_RasterScanMovePattern),
        ("TriggerInPolarity", MOT_TriggerPolarity),
        ("TriggerMode", MOT_RasterScanMoveTriggerMode),
        ("TriggerSource", MOT_TriggerInputSource),
    ]


class MOT_StageAxisParameters(Structure):
    _fields_ = [
        ("stageID", c_long),
        ("axisID", c_long),
        ("partNumber", 16 * c_char),
        ("serialNumber", c_ulong),
        ("countsPerUnit", c_ulong),
        ("minPosition", c_int),
        ("maxPosition", c_int),
        ("maxAcceleration", c_int),
        ("maxDecceleration", c_int),
        ("maxVelocity", c_int),
        ("reserved1", c_long),
        ("reserved2", c_long),
        ("reserved3", c_long),
        ("reserved4", c_long),
        ("reserved5", c_ulong),
        ("reserved6", c_ulong),
        ("reserved7", c_ulong),
        ("reserved8", c_ulong),
    ]


class MOT_TriggerIOConfigParameters(Structure):
    _fields_ = [
        ("CycleCount", c_long),
        ("InputSource", MOT_TriggerInputSource),
        ("IntervalFwd", c_long),
        ("IntervalRev", c_long),
        ("PulseCountFwd", c_long),
        ("PulseCountRev", c_long),
        ("PulseWidth", c_long),
        ("reserved1", c_long),
        ("reserved2", c_long),
        ("reserved3", c_long),
        ("reserved4", c_long),
        ("StartPositionFwd", c_long),
        ("StartPositionRev", c_long),
        ("TriggerInMode", MOT_TriggerInputConfigModes),
        ("TriggerInPolarity", MOT_TriggerPolarity),
        ("TriggerOutMode", MOT_TriggerOutputConfigModes),
        ("TriggerOUtPolarity", MOT_TriggerPolarity),
    ]


class MOT_VelocityProfileParameters(Structure):
    _fields_ = [
        ("mode", MOT_VelocityProfileModes),
        ("jerk", c_ulong),
        ("notUsed", c_long),
        ("lastNotUsed", c_long),
    ]


class MPC_IOSettings(Structure):
    _fields_ = [
        ("ADCSpeedValue", c_uint),
        ("digIO1OperMode", MPC_IOModes),
        ("digIO1PulseWdith", c_uint),
        ("digIO1SignalMode", MPC_SignalModes),
        ("digIO2OperMode", MPC_IOModes),
        ("digIO2PulseWdith", c_uint),
        ("digIO2SignalMode", MPC_SignalModes),
        ("reserved1", c_int),
        ("reserved2", c_uint),
        ("transitTime", c_uint),
    ]


class NT_CircleDiameterLUT(Structure):
    _fields_ = [
        ("LUT_Diameter1", c_long),
        ("LUT_Diameter2", c_long),
        ("LUT_Diameter3", c_long),
        ("LUT_Diameter4", c_long),
        ("LUT_Diameter5", c_long),
        ("LUT_Diameter6", c_long),
        ("LUT_Diameter7", c_long),
        ("LUT_Diameter8", c_long),
        ("LUT_Diameter9", c_long),
        ("LUT_Diameter10", c_long),
        ("LUT_Diameter11", c_long),
        ("LUT_Diameter12", c_long),
        ("LUT_Diameter13", c_long),
        ("LUT_Diameter14", c_long),
        ("LUT_Diameter15", c_long),
        ("LUT_Diameter16", c_long),
    ]


class NT_CircleParameters(Structure):
    _fields_ = [
        ("algorithmAdjustment", NT_CircleAdjustment),
        ("diameter", c_long),
        ("maxDiameter", c_long),
        ("minDiameter", c_long),
        ("mode", NT_CircleDiameterMode),
        ("samplesPerRevolution", c_long),
    ]


class NT_GainParameters(Structure):
    _fields_ = [
        ("controlMode", c_long),
        ("gain", c_short),
    ]


class NT_HVComponent(Structure):
    _fields_ = [
        ("horizontalComponent", c_long),
        ("verticalComponent", c_long),
    ]


class NT_IOSettings(Structure):
    _fields_ = [
        ("lowVoltageOutputRoute", NT_OutputVoltageRoute),
        ("lowVoltageOutputRange", NT_VoltageRange),
        ("notYetInUse", c_long),
        ("unused", c_long),
    ]


class NT_LowPassFilterParameters(Structure):
    _fields_ = [
        ("param1", NT_LowPassFrequency),
        ("notUsed1", NT_LowPassFrequency),
        ("notUsed2", NT_LowPassFrequency),
        ("notUsed3", NT_LowPassFrequency),
        ("notUsed4", NT_LowPassFrequency),
    ]


class NT_TIARangeParameters(Structure):
    _fields_ = [
        ("changeToOddOrEven", NT_OddOrEven),
        ("downLimit", c_long),
        ("mode", NT_TIARangeMode),
        ("newRange", NT_TIARangeMode),
        ("settleSamples", c_short),
        ("upLimit", c_long),
    ]


class NT_TIAReading(Structure):
    _fields_ = [
        ("absoluteReading", c_float),
        ("relativeReading", c_long),
        ("selectedRange", NT_TIARangeMode),
        ("underOrOverRead", NT_UnderOrOver),
    ]


class PDXC2_ClosedLoopParameters(Structure):
    _fields_ = [
        ("Acceleration", c_uint32),
        ("Differential", c_uint32),
        ("Integral", c_uint32),
        ("Proportional", c_uint32),
        ("RefSpeed", c_uint32),
    ]


class PDXC2_JogParameters(Structure):
    _fields_ = [
        ("ClosedLoopAcceleration", c_int32),
        ("CloseLoopSpeed", c_int32),
        ("CloseLoopStepSize", c_int32),
        ("JogMode", PZ_JogModes),
        ("OpenLoopAcceleration", c_int32),
        ("OpenLoopStepRate", c_int32),
        ("OpenLoopStepSize", c_int32),
    ]


class PDXC2_OpenLoopMoveParameters(Structure):
    _fields_ = [
        ("StepSize", c_int32),
    ]


class PDXC2_Status(Structure):
    _fields_ = [
        ("Position", c_int32),
        ("StatusBit", c_int32),
        ("Unused", c_int32),
    ]


class PDXC2_TriggerParams(Structure):
    _fields_ = [
        ("analongInGain", c_float),
        ("analogInOffset", c_float),
        ("analogOutGain", c_float),
        ("analogOutOffset", c_float),
        ("fallFixedStep", c_int32),
        ("fallPosition1", c_int32),
        ("fallPosition2", c_int32),
        ("riseFixedStep", c_int32),
        ("risePosition1", c_int32),
        ("risePosition2", c_int32),
    ]


class PPC_IOSettings(Structure):
    _fields_ = [
        ("controlSrc", PPC_IOControlMode),
        ("feedbackPolarity", PCC_FeedbackPolarity),
        ("feedbackSrc", PCC_IOFeedbackSourceDefinition),
        ("FPBrightness", PCC_DisplayIntensity),
        ("monitorOPBandwidth", PCC_IOOutputBandwidth),
        ("monitorOPSig", PCC_IOOutputMode),
    ]


class PPC_NotchParams(Structure):
    _fields_ = [
        ("filter1Fc", c_float),
        ("filter1Q", c_float),
        ("filter2Fc", c_float),
        ("filter2Q", c_float),
        ("filterNo", PCC_NotchFilterChannel),
        ("notchFilter1On", PCC_NotchFilterState),
        ("notchFilter2On", PCC_NotchFilterState),
    ]


class PPC_PIDConsts(Structure):
    _fields_ = [
        ("PIDConstsD", c_float),
        ("PIDConstsDFc", c_float),
        ("PIDConstsI", c_float),
        ("PIDConstsP", c_float),
        ("PIDDerivFilterOn", PCC_DerivFilterState),
        ("PIDIndex", c_byte),
    ]


class PPC_PIDCriteria(Structure):
    _fields_ = [
        ("criterialD", c_uint8),
        ("PIDConstIndex", c_uint16),
        ("priority", c_uint16),
        ("targetErrorWindow", c_uint16),
        ("unusedByte", c_uint8),
        ("wReservedPart1", c_long),
        ("wReservedPart2", c_long),
        ("wReservedPart3", c_long),
    ]


class PZ_FeedbackLoopConstants(Structure):
    _fields_ = [
        ("integratTerm", c_short),
        ("proportionalTerm", c_short),
    ]


class PZ_LUTWaveParameters(Structure):
    _fields_ = [
        ("cycleLength", c_short),
        ("LUTValueDelay", c_uint),
        ("mode", PZ_OutputLUTModes),
        ("numCycles", c_uint),
        ("numOutTriggerRepeat", c_short),
        ("outTriggerDuration", c_uint),
        ("outTriggerStart", c_short),
        ("postCycleDelay", c_uint),
        ("preCycleDelay", c_uint),
    ]


class QD_ClosedLoopPosition(Structure):
    _fields_ = [
        ("x", c_int16),
        ("y", c_int16),
    ]


class QD_KPA_DigitalIO(Structure):
    _fields_ = [
        ("wDigOPs", c_long),
        ("wReservedPart1", c_long),
        ("wReservedPart2", c_long),
        ("wReservedPart3", c_long),
        ("wReservedPart4", c_long),
        ("wReservedPart5", c_long),
        ("wReservedPart6", c_long),
    ]


class QD_KPA_TrigIOConfig(Structure):
    _fields_ = [
        ("trig1DiffThreshold", c_long),
        ("trig1Mode", QD_KPA_TrigModes),
        ("trig1Polarity", QD_KPA_TrigPolarities),
        ("trig1SumMax", c_long),
        ("trig1SumMin", c_long),
        ("trig2DiffThreshold", c_long),
        ("trig2Mode", QD_KPA_TrigModes),
        ("trig2Polarity", QD_KPA_TrigPolarities),
        ("trig2SumMax", c_long),
        ("trig2SumMin", c_long),
        ("wReservedPart1", c_long),
        ("wReservedPart2", c_long),
        ("wReservedPart3", c_long),
        ("wReservedPart4", c_long),
        ("wReservedPart5", c_long),
        ("wReservedPart6", c_long),
    ]


class QD_LoopParameters(Structure):
    _fields_ = [
        ("differentialGain", c_float),
        ("integralGain", c_float),
        ("lowPassfilterCutOffFreq", c_float),
        ("lowPassFilerEnabled", QD_FilterEnable),
        ("notchFilterCenterFrequency", c_float),
        ("notchFilterEnabled", QD_FilterEnable),
        ("notchFilterQ", c_float),
        ("proportionalGain", c_float),
    ]


class QD_LowPassFilterParameters(Structure):
    _fields_ = [
        ("lowPassFilterCutOffFreq", c_float),
        ("lowPassFilterEnabled", QD_FilterEnable),
    ]


class QD_NotchFilterParameters(Structure):
    _fields_ = [
        ("notchFilterCenterFrequency", c_float),
        ("notchFilterEnable", QD_FilterEnable),
        ("notchFilterQ", c_float),
    ]


class QD_PIDParameters(Structure):
    _fields_ = [
        ("differentialGain", c_float),
        ("integralGain", c_float),
        ("proportionalGain", c_float),
    ]


class QD_Position(Structure):
    _fields_ = [
        ("x", c_int16),
        ("y", c_int16),
    ]


class QD_PositionDemandParameters(Structure):
    _fields_ = [
        ("lowVoltageOutputRoute", QD_LowVoltageRoute),
        ("maxXdemand", c_int16),
        ("maxYdemand", c_int16),
        ("minXdemand", c_int16),
        ("minYdemand", c_int16),
        ("openLoopOptions", QD_OpenLoopHoldValues),
        ("xFeedBackSignedGain", c_int16),
        ("yFeedBackSignedGain", c_int16),
    ]


class QD_Readings(Structure):
    _fields_ = [
        ("demandPos", QD_Position),
        ("posDifference", QD_Position),
        ("sum", c_long),
    ]


class SC_CycleParameters(Structure):
    _fields_ = [
        ("closeTime", c_uint),
        ("numCycles", c_uint),
        ("opeTime", c_uint),
    ]


class TC_LoopParameters(Structure):
    _fields_ = [
        ("differentialGain", c_ushort),
        ("integralGain", c_ushort),
        ("proportionalGain", c_ushort),
    ]


class TLI_DeviceInfo(Structure):
    _fields_ = [
        ("typeID", c_ulong),
        ("description", 65 * c_char),
        ("serialNo", 9 * c_char),
        ("PID", c_ulong),
        ("isKnownType", c_bool),
        ("motorType", MOT_MotorTypes),
        ("isPiezoDevice", c_bool),
        ("isLaser", c_bool),
        ("isCustomType", c_bool),
        ("isRack", c_bool),
        ("maxChannels", c_short),
    ]


class TLI_HardwareInformation(Structure):
    _fields_ = [
        ("serialNumber", c_ulong),
        ("modelNumber", 8 * c_char),
        ("type", c_long),
        ("firmwareVersion", c_ulong),
        ("notes", 48 * c_char),
        ("deviceDependantData", 12 * c_byte),
        ("hardwareVersion", c_long),
        ("modificationState", c_long),
        ("numChannels", c_short),
    ]


class TPZ_IOSettings(Structure):
    _fields_ = [
        ("hubAnalogueInput", HubAnalogueModes),
        ("maximumOutputVoltage", c_short),
    ]


class TSG_IOSettings(Structure):
    _fields_ = [
        ("displayMode", TSG_DisplayModes),
        ("forceCalibration", c_uint),
        ("futureUse", c_long),
        ("hubAnalogOutput", TSG_Hub_Analogue_Modes),
        ("notYetInUse", c_long),
    ]
